@@ -22,22 +22,41 @@ xmlise (c:cs) = c:(xmlise cs)
 showfname (NS s n) = showfname n
 showfname x = showuser x
 
-writeXMLDocs :: FilePath -> Program -> String -> IO ()
-writeXMLDocs fn prog inf
-  = do let str = "<module>\n" ++ mkXMLDocs prog inf ++ "</module>\n"
+data XMLDocType = XTySyn | XData | XFunc | XExcept
+    deriving (Eq, Ord)
+data XMLDoc = XMLDoc {
+                      xmlty :: XMLDocType,
+                      xmlfn :: String,
+                      xml :: String
+                     }
+    deriving (Eq, Ord)
+        
+writeXMLDocs :: FilePath -> Program -> String -> String -> IO ()
+writeXMLDocs fn prog inf mdocstr
+  = do let str = "<module>\n" ++ 
+                 "<modsummary>\n" ++ mdocstr ++ "</modsummary>\n" ++
+                 concat (map xml (sort (mkXMLDocs prog inf))) ++ 
+                 "</module>\n"
        writeFile fn str
 
-mkXMLDocs :: Program -> String -> String
-mkXMLDocs [] f = ""
+mkXMLDocs :: Program -> String -> [XMLDoc]
+mkXMLDocs [] f = []
 mkXMLDocs ((FunBind (f,l,n,_,fopts,Defined def) comm ty):xs) inf 
        | inf == f && (elem Public fopts) =
-	   fundoc f l n ty def comm ++ "\n" ++ mkXMLDocs xs inf
+	   (XMLDoc XFunc (cname n " "++(mangling ty))
+                         (fundoc f l n ty def comm fopts)):(mkXMLDocs xs inf)
 mkXMLDocs ((DataDecl f l dopts n tys cons comm):xs) inf 
-       | inf == f =
-	   datadoc f l n tys cons comm ++ "\n" ++ mkXMLDocs xs inf
+       | inf == f && (elem DExport dopts) && ((elem DPublic dopts) || (elem DAbstract dopts))  =
+	   (XMLDoc XData (show n) 
+                         (datadoc f l n tys cons comm dopts)):(mkXMLDocs xs inf)
+mkXMLDocs ((ExceptDecl f l n tys comm):xs) inf 
+       | inf == f = 
+           (XMLDoc XExcept (show n)
+                           (exceptdoc f l n tys comm)):(mkXMLDocs xs inf)
 mkXMLDocs ((TySyn (f, l, n, ps, ty, True)):xs) inf 
        | inf == f =
-	   tysyndoc f l n ps ty ++ "\n" ++ mkXMLDocs xs inf
+	   (XMLDoc XTySyn (show n) 
+                          (tysyndoc f l n ps ty)):(mkXMLDocs xs inf)
 mkXMLDocs (x:xs) inf = mkXMLDocs xs inf
 
 data FunInfo = Fun Name -- Name
@@ -101,7 +120,7 @@ ddetails [] = ""
 ddetails ((TypeSyn n ps syn):xs) =
     "<hr><a name=\""++showfname n++"\">\n" ++
     "<h3>type " ++ showuser n ++ showPlist ps ++ "</h3>\n" ++
-    "<p><strong> = " ++ syn ++ "</strong></p>\n" ++
+    "<p><strong> = " ++ xmlise syn ++ "</strong></p>\n" ++
     ddetails xs
 ddetails ((Data n ps cons comm):xs) = 
     "<hr><a name=\""++showfname n++"\">\n" ++
@@ -140,50 +159,75 @@ mkHTMLDocs ((TySyn (f,l,n,ps,ty,True)):xs) inf
 	   (fi,(tysynhtml f l n ps ty):di)
 mkHTMLDocs (x:xs) inf = mkHTMLDocs xs inf
 
-fundoc :: String -> Int -> Name -> Type -> Expr Name -> String -> String
-fundoc f l n ty def comm = "<fun><name>"++showfname n++"</name>\n"++
-			   showcomment comm++
+exceptdoc :: String -> Int -> Name -> [Type] -> String -> String
+exceptdoc f l n tys comm = "<except><name>"++showuser n++"</name>\n" ++
+                           showcomment comm++
 			   "<location><file>"++f++"</file>"++
 			   "<line>"++show l++"</line></location>\n"++
-			   "<return>"++xmlise (show (rettype ty))++
-			   "</return>\n"++
-			   listargs def (argtype ty) ++ 
---			   "<comment>"++show comm++"</comment>"++
-			   "</fun>\n"
+                           showexcargs tys++
+                           "</except>"
+    where showcomment "" = "" 
+	  showcomment comm = "<comment>"++comm++"</comment>\n"
+	  showexcargs [] = ""
+          showexcargs (a:as) = "<arg><type>" ++
+	                       xmlise (show a) ++ "</type></arg>" ++ 
+                               showexcargs as
+
+
+fundoc :: String -> Int -> Name -> Type -> Expr Name -> String -> [FOpt] -> String
+fundoc f l n ty def comm fopts = "<fun><name>"++showfname n++"</name>\n"++
+                                 showdepr (elem DeprecatedFn fopts)++
+			         showcomment comm++
+			         "<location><file>"++f++"</file>"++
+			         "<line>"++show l++"</line></location>\n"++
+			         "<return>"++xmlise (show (rettype ty))++
+			         "</return>\n"++
+			         listargs def (argtype ty) (deftype ty) ++ 
+                                 --			   "<comment>"++show comm++"</comment>"++
+			         "</fun>\n"
    where showcomment "" = ""
 	 showcomment comm = "<comment>"++comm++"</comment>\n"
+         showdepr False = ""
+         showdepr True = "<deprecated/>\n"
   	 rettype (Fn _ _ ty) = ty
 	 argtype (Fn _ args _) = args
+	 deftype (Fn defs _ _) = defs
 
-         listargs (Lambda ivs tys _) args = la' tys args (fArgs ivs tys)
-         listargs (Annotation _ e) args = listargs e args
-	 listargs _ _ = ""
-	 la' [] [] _ = ""
-	 la' ((n,_):tys) (ty:tys') varargs = 
+         listargs (Lambda ivs tys _) args defs = la' tys args defs (fArgs ivs tys)
+         listargs (Annotation _ e) args defs = listargs e args defs
+	 listargs _ _ _ = ""
+	 la' [] [] [] _ = ""
+	 la' ((n,_):tys) (ty:tys') (d:ds) varargs = 
 	     "<arg passby=\"" ++
                (if elem n varargs then "var" else "copy") ++ 
              "\"><argname>" ++ showuser n ++ "</argname><type>" ++
-	     xmlise (show ty) ++ "</type></arg>" ++ la' tys tys' varargs
+	     xmlise (show ty) ++ "</type>" ++ showdef d ++"</arg>" ++ la' tys tys' ds varargs
+
+	 showdef Nothing = ""
+	 showdef (Just d) = "<defval>" ++ (stripat (showexp d)) ++ "</defval>"
+	 stripat ('@':xs) = xs
+	 stripat xs = xs
+
 
 tysyndoc :: String -> Int -> Name -> [Name] -> Type -> String
 tysyndoc f l n ps ty = "<tysyn><name>"++showuser n++"</name>\n"++
 		       "<location><file>"++f++"</file>"++
-		       "<line>"++show l++"</line></location>\n"++
-		       listparams ps ++ "\n" ++
+		       "<line>"++show l++"</line></location>\n" ++
+		       listparams ps ++ (if not (null ps) then "\n" else "") ++
                        "<syn>"++show ty ++ "</syn>" ++
-                       "</tysyn>"
+                       "</tysyn>\n"
    where listparams [] = ""
 	 listparams (p:ps) = "<param>"++showuser p++"</param>" ++ listparams ps
 
-datadoc :: String -> Int -> Name -> [Type] -> [ConDecl] -> String -> String
-datadoc f l n tys cons comm = "<data><name>"++showuser n++"</name>\n"++
-			      showcomment comm ++
-			      "<location><file>"++f++"</file>"++
-			      "<line>"++show l++"</line></location>\n"++
-			      listparams tys ++ "\n" ++
-			      listcons cons ++
---			      "<comment>"++show comm++"</comment>"++
-			      "</data>\n"
+datadoc :: String -> Int -> Name -> [Type] -> [ConDecl] -> String -> [DOpt] -> String
+datadoc f l n tys cons comm dopts = "<data><name>"++showuser n++"</name>\n"++
+			            showcomment comm ++
+			            "<location><file>"++f++"</file>"++
+			            "<line>"++show l++"</line></location>\n"++
+			            listparams tys ++ "\n" ++
+			            listcons cons ++
+                                    -- "<comment>"++show comm++"</comment>"++
+			            "</data>\n"
    where showcomment "" = ""
 	 showcomment comm = "<comment>"++comm++"</comment>\n"
          listparams [] = ""
@@ -195,15 +239,17 @@ datadoc f l n tys cons comm = "<data><name>"++showuser n++"</name>\n"++
 	 argtype (Fn _ args _) = args
          showcon (Con n ty ns _) =
 	     "<con><conname>"++showfname n++"</conname>"++
-	     showconargs (argtype ty) ns ++
+	     showconargs (argtype ty) ns (elem DAbstract dopts) ++
 	     "</con>\n"
-	 showconargs [] [] = ""
-	 showconargs (a:as) (None:ns) = 
+
+         showconargs _ _ True = ""
+         showconargs [] [] False = ""
+	 showconargs (a:as) (None:ns) False = 
 	     "<arg><type>" ++
-	     xmlise (show a) ++ "</type></arg>" ++ showconargs as ns
-	 showconargs (a:as) (n:ns) = 
+	     xmlise (show a) ++ "</type></arg>" ++ showconargs as ns False
+	 showconargs (a:as) (n:ns) False = 
 	     "<arg><argname>"++showuser n++"</argname><type>" ++
-	     xmlise (show a) ++ "</type></arg>" ++ showconargs as ns
+	     xmlise (show a) ++ "</type></arg>" ++ showconargs as ns False
 
 funhtml :: String -> Int -> Name -> Type -> Expr Name -> String -> FunInfo
 funhtml f l n ty def comm = Fun n synopsis comm
