@@ -8,6 +8,7 @@
 
 module CodegenCPP where
 
+import Options
 import TAC
 import Language
 import IO
@@ -15,8 +16,9 @@ import Debug.Trace
 import Lib
 
 data Output = RawOutput String
-	    | FNOutput (Name, Type, String)
+	    | FNOutput (Name, Type, String, [(ArgType, Var)], String)
 	    | ExternOutput (Name, Type)
+            | ExcOutput Name Bool
 	    | GlobOutput Int
   deriving Show
 
@@ -30,10 +32,10 @@ writeCf domain lds xs out = do mprog <- getmain lds domain
 
 writeC :: Name -> -- Module name
 	  InputType -> [FilePath] -> Context -> 
-	  [CompileResult] -> Handle -> IO ()
-writeC mod domain lds ctxt xs out 
-    = do mprog <- getmain mod lds ctxt domain
-	 header <- getheader mod lds ctxt domain
+	  [CompileResult] -> Handle -> Options -> IO ()
+writeC mod domain lds ctxt xs out copts
+    = do mprog <- getmain mod lds ctxt domain copts
+	 header <- getheader mod lds ctxt domain copts
 	 let str = ((writeout header mprog).writecpp) xs
 	 hPutStr out str
 	 hClose out
@@ -48,29 +50,35 @@ writeout' ((RawOutput str):xs) = str ++ "\n" ++ writeout' xs
 writeout' ((ExternOutput (f, ty)):xs) 
     = "void " ++ (show f) ++ mangling ty ++ "(VMState* vm);\n" 
 				  ++ writeout' xs
-writeout' ((FNOutput (f, ty, def)):rest) 
-    = "void " ++ (show f) ++ mangling ty ++ "(VMState* vm){\n" ++ def ++ "}\n"
+writeout' ((ExcOutput nm defhere):xs)
+    = (if defhere then "" else "extern ") ++
+         "char* " ++ show nm ++ 
+      (if defhere then " = \"" ++ showuser nm ++"\"" else "") ++
+         ";\n" ++ writeout' xs
+writeout' ((FNOutput (f, ty, pop, args, def)):rest) 
+    = "void " ++ (show f) ++ mangling ty ++ "(VMState* vm){\n" ++ pop ++ "\n" ++ def ++ "}\n\n"
       ++ writeout' rest 
 writeout' ((GlobOutput n):rest)
     = "DECLGLOBAL("++show n++");\n"
       ++ writeout' rest
 
 writedecls [] = ""
-writedecls ((FNOutput (f, ty, def)):rest) 
+writedecls ((FNOutput (f, ty, _, _, _)):rest) 
     = "void " ++ (show f) ++ mangling ty ++ "(VMState* vm);\n"
       ++ writedecls rest 
 writedecls (_:xs) = writedecls xs
 
-getmain,getheader :: Name -> [FilePath] -> Context -> InputType -> IO String
-getmain mod lds ctxt Program = 
+getmain,getheader :: Name -> [FilePath] -> Context -> InputType -> Options -> IO String
+getmain mod lds ctxt (Program _) copts = 
    do str <- findFile lds "startup.vcc"
-      str' <- replaceDefs mod ctxt ["__start","__panic"] str 
+      str' <- replaceDefs mod ctxt ["__start","__panic"] str copts
       return str'
-getmain mod lds ctxt Shebang = 
+getmain mod lds ctxt Shebang copts = 
    do str <- findFile lds "startup.vcc"
-      str' <- replaceDefs mod ctxt ["__start","__panic"] str 
+      str' <- replaceDefs mod ctxt ["__start","__panic"] str copts
       return str'
-getmain mod lds ctxt Module = return ""
+getmain mod lds ctxt Module copts = return ""
+{-
 getmain mod lds ctxt Webapp = 
    do str <- findFile lds "startup.vcc"
       str' <- replaceDefs mod ctxt ["__start","__panic"] str 
@@ -79,6 +87,7 @@ getmain mod lds ctxt Webprog =
    do str <- findFile lds "startup.vcc"
       str' <- replaceDefs mod ctxt ["__start","__panic"] str 
       return str'
+-}
 
 -- getmain mod lds ctxt Program = 
 --    do str <- findFile lds "program.vcc"
@@ -95,22 +104,26 @@ getmain mod lds ctxt Webprog =
 -- 				   "initWebApp","flush","IllegalHandler"] str 
 --        return str'
 
-getheader mod lds ctxt _ = 
+getheader mod lds ctxt _ copts = 
    do hf <- findFile lds "header.vcc"
       return $ hf ++ "\nValue** globaltable"++show mod++"=NULL;\n\n"
 
 getStartup :: InputType -> [FilePath] -> IO String
-getStartup Program lds = findFile lds "program.ks"
-getStartup Shebang lds = findFile lds "program.ks"
-getStartup Webapp lds = findFile lds "webapp.ks"
-getStartup Webprog lds = findFile lds "webprog.ks"
+getStartup (Program s) lds = do contents <- findFile lds (s++".ks")
+                                return $ "%line 0 %file \"" ++ s ++ ".ks\"\n" 
+                                         ++ contents
+getStartup Shebang lds = do contents <- findFile lds "program.ks"
+                            return $ "%line 0 %file \"program.ks\"\n" 
+                                         ++ contents
+-- getStartup Webapp lds = findFile lds "webapp.ks"
+-- getStartup Webprog lds = findFile lds "webprog.ks"
 getStartup _ _ = return ""
 
-replaceDefs :: Name -> Context -> [String] -> String -> IO String
-replaceDefs mod ctxt [] x = return x
-replaceDefs mod ctxt (x:xs) str = 
-    do newstr <- replaceDefs mod ctxt xs str
-       (fname,_) <- ctxtlookup mod (UN x) ctxt Nothing
+replaceDefs :: Name -> Context -> [String] -> String -> Options -> IO String
+replaceDefs mod ctxt [] x copts = return x
+replaceDefs mod ctxt (x:xs) str copts = 
+    do newstr <- replaceDefs mod ctxt xs str copts
+       (fname,_) <- ctxtlookup mod (UN x) ctxt Nothing copts
        let str' = replace ("%"++x) (show fname) newstr
        return str'
 
@@ -134,8 +147,9 @@ findFile (x:xs) path
 writecpp :: [CompileResult] -> [Output]
 writecpp [] = []
 writecpp ((RawCode str):xs) = (RawOutput str):(writecpp xs)
-writecpp ((ByteCode (n, ty, def)):xs) = {- trace (show n ++ show def) $-} ((FNOutput (n, ty, cpp def)):writecpp xs)
+writecpp ((ByteCode (n, ty, (Code pop args def))):xs) = {- trace (show n ++ show def) $-} ((FNOutput (n, ty, cpp pop, args, cpp def)):writecpp xs)
 writecpp ((ExternDef (n,ty)):xs) = (ExternOutput (n, ty)):writecpp xs
+writecpp ((ExcCode nm here):xs) = (ExcOutput nm here):(writecpp xs)
 writecpp ((GlobCode n):xs) = (GlobOutput n):writecpp xs
 
 cpp :: [TAC] -> String
@@ -189,16 +203,22 @@ instr (RTMPSET tmp val) = "t"++show tmp++"="++show val
 instr (CALL v) = "CALLFUN("++show (fst v)++")"
 instr (CALLNAME f) = "CALL("++f++")"
 instr CALLTOP = "CALLTOP"
+instr (FASTCALL nm args) = nm ++ "(vm, " ++ showlist args ++ ")"
+    where showlist [] = ""
+	  showlist [x] = show x
+	  showlist (x:xs) = show x ++ "," ++ show xs
 instr (TAILCALL v) = "TAILCALLFUN("++show (fst v)++")"
 instr (TAILCALLNAME f) = "TAILCALL("++f++")"
 instr TAILCALLTOP = "TAILCALLTOP"
 instr (CLOSURE n i arity) 
     = "CLOSURE("++ n ++","++show i++"," ++ show arity ++")"
 instr (CLOSURELOC v i) = "CLOSURELOC("++ show (fst v) ++","++show i++")"
+instr (CLOSURETOP i) = "CLOSURETOP("++show i++")"
 instr (FOREIGNCALL n lib ty args) = mkfcall n lib ty args 
 instr (MKCON t i) = "MKCON("++show t ++ "," ++ show i ++ ")"
 instr (MKCONZERO t) = "MKCONZERO("++show t ++ ")"
 instr MKEXCEPT = "MKEXCEPT"
+instr (MKNEWEXCEPT n a) = "MKNEWEXCEPT(" ++ show n ++ ", " ++ show a++")"
 instr EQEXCEPT = "EQEXCEPT"
 instr NEEXCEPT = "NEEXCEPT"
 instr EQSTRING = "EQSTRING"
@@ -209,10 +229,12 @@ instr (JEQSTRING l) = "JEQSTRING("++show l++")"
 instr (JNESTRING l) = "JNESTRING("++show l++")"
 instr (JEQSTRINGW l v) = "JEQSTRINGW("++show l++",L"++show v++")"
 instr (JNESTRINGW l v) = "JNESTRINGW("++show l++",L"++show v++")"
+instr (JEXNE ex l) = "JEXNE("++show ex++","++show l++")"
 instr (GETVAL v) = "GETVAL(t"++show v++")"
 instr (GETRVAL v) = "GETRVAL(t"++show v++")"
 instr GETINDEX = "GETINDEX"
 instr (PROJARG a t) = "PROJARG("++ show a ++ "," ++ show t ++ ")"
+instr (EXPROJARG a t) = "EXPROJARG("++ show (fst a) ++ "," ++ show t ++ ")"
 instr (INFIX t Divide x y) = "INTDIV(" ++ tmp t ++ "," ++ 
 			 tmp x ++ "," ++ tmp y ++ ")"
 instr (INFIX t op x y) = "INTINFIX(" ++ tmp t ++ "," ++ printOp op ++ "," ++ 
