@@ -19,9 +19,12 @@ import APIDocs
 import Options
 import LambdaLift
 import Optimise
+import Propagate
+import Inliner
 import CallGraph
 import PureFun
 import Lib
+import Errors
 
 import System
 import System.Directory
@@ -55,40 +58,49 @@ compileFile prtype modname fn extra opts =
        compile newroot libdirs opts pt extra
 
 outputfile Module mod = showuser mod ++ ".o"
-outputfile Program mod = showuser mod
+-- TMP HACK: This should probably be a %extension "cgi" directive in the .ks
+outputfile (Program "webapp") mod = showuser mod ++ ".cgi"
+outputfile (Program "cgi") mod = showuser mod ++ ".cgi"
+outputfile (Program _) mod = showuser mod
 outputfile Shebang mod = showuser mod
-outputfile Webapp mod = showuser mod ++ ".cgi"
-outputfile Webprog mod = showuser mod ++ ".cgi"
+-- outputfile Webapp mod = showuser mod ++ ".cgi"
+-- outputfile Webprog mod = showuser mod ++ ".cgi"
 
 compile :: String -> [FilePath] -> Options -> 
 	   (Result ParseResult) -> [String] -> IO ()
 compile root libdirs opts (Failure err file line) extra 
     = do putStrLn err
 	 exitWith (ExitFailure 1)
-compile root libdirs opts prog@(Success (PR t mod xs)) extrain = 
+compile root libdirs opts prog@(Success (PR t mod xs mdocstr)) extrain = 
     do dump (dumpraw opts) xs
        let extra = if t == Shebang then [] else extrain
-       case (inferAll (UN root) [] [] [] [] [] (dumpeqns opts) xs) of
-	    Success (ok,ctxt,_,_,_) -> do let ok' = if (doprelude opts)
-						       then (Imported "Prelude"):ok
-						       else ok
-					  comp' t root libdirs opts ctxt
-					       (nub (lambdalift ok')) extra mod
-	    Failure err f l -> do putStrLn err
+       case (inferAll (UN root) empty [] [] empty [] empty [] (dumpeqns opts) xs opts) of
+	    Success (ok,[],ctxt,_,_,_,_) -> 
+                do let ok' = if (doprelude opts)
+			       then (Imported "Prelude"):ok
+			       else ok
+                   comp' t root libdirs opts ctxt (nub (lambdalift ok')) extra mod mdocstr
+	    Success (ok,errs,ctxt,_,_,_,_) -> 
+                               do mapM_ reportError errs
+				  exitWith (ExitFailure 1)
+	    Failure err f l -> do reportError err
 				  exitWith (ExitFailure 1)
   where	dump True raw = putStr $ concat (map ((\x -> x++"\n\n").show) raw)
 	dump False raw = return ()
 
-comp' t root libdirs opts ctxt ok extra mod = do
-       optlist <- useOpts allOptNames
-       let optimised = if (noopts opts) then findPure (sortprog ok)
-	                  else (findPure.sortprog) (optAll optlist ok)
+comp' t root libdirs opts ctxt ok extra mod mdocstr = do
+       let globbed = findPure (sortprog (addGlobInit ok mod))
+
+       let optLevel = if (noopts opts) then 0 else 1
+       let inls = getInlinable globbed
+       let optimised = runOpts optLevel globbed
+
        if (dumpcg opts) 
 	  then putStrLn $ dumpCG (makeCG optimised)
 	  else return ()
        dumppt (dumptree opts) optimised
        case (compileAll optimised mod) of
-          Failure err f l -> do putStrLn err
+          Failure err f l -> do reportError err
 				exitWith (ExitFailure 1)
 	  Success comp -> do
 	     let dynlinks = getdynlinks opts []
@@ -99,26 +111,34 @@ comp' t root libdirs opts ctxt ok extra mod = do
 	     let ifile = root ++ ".ki"
 	     let xfile = root ++ ".xml"
 	     let hfile = root ++ ".html"
-	     doWriteIface t ifile optimised
-	     doWriteXMLDocs (xmldocs opts) xfile optimised (root++".k")
+	     doWriteIface t inls ifile optimised
+	     doWriteXMLDocs (xmldocs opts) xfile optimised (root++".k") mdocstr
 	     doWriteHTMLDocs (htmldocs opts) hfile optimised (root++".k")
 --       putStrLn (show ok)
 --       let name = tmpdir++root++".vcc"
 --       putStrLn $ tmpdir++ " is tmp"
 	     (tmpn,tmph) <- tempfile
 --       putStrLn $ tmpn++ " is file"
-	     aeskey <- case getseed opts of
-		    Nothing -> mkaeskey 0
-		    (Just x) -> mkaeskey (hash x)
-	     ivec <- case getseed opts of
-		    Nothing -> mkivec 0
-		    (Just x) -> mkivec (hash (x++"ivec"))
+	     aeskey <- case t of
+                      Module -> return (RawCode $ "")
+                      _ -> case getseed opts of
+		                Nothing -> mkaeskey 0
+		                (Just x) -> mkaeskey (hash x)
+	     ivec <- case t of 
+                     Module -> return (RawCode $ "")
+                     _ -> case getseed opts of
+		               Nothing -> mkivec 0
+		               (Just x) -> mkivec (hash (x++"ivec"))
 	     dump (dumptac opts) comp
     -- FIXME: (UN root) really ought to be mod, but I need to fix the
     -- parser to update the module name when it discovers what it is.
 	     writeC (UN root) t libdirs ctxt 
-			(addfnmap t optimised comp [aeskey,ivec]) tmph
+			(addfnmap t optimised comp [aeskey,ivec]) tmph opts
 	     let doprofile = if (profile opts) then "-g -pg " else ""
+	     let dostatic = if (static opts) then case t of
+                                                         Module -> ""
+                                                         _ -> " -static -lpthread -lc " 
+                                                         else ""
 	     let cmd = "g++ " ++ nocheck ++ addc t ++ " " ++ 
 		       " -O2 " ++ 
                        "-fno-optimize-sibling-calls " ++ 
@@ -129,7 +149,7 @@ comp' t root libdirs opts ctxt ok extra mod = do
 		       outputfile t mod ++ " " ++ (unwords extra) ++ " " ++ 
 		       incl libdirs ++ 
 		       showlist ofiles ++ " " ++ showlist linkopts ++ " " ++
-		       dolink t
+		       dolink t ++ dostatic
 	     case showgcc opts of
 		 True -> putStrLn cmd
 		 False -> return ()
@@ -146,16 +166,16 @@ comp' t root libdirs opts ctxt ok extra mod = do
         libopts = concat $ map (" -L"++) libpath
 	libvm = if (nortchecks opts) then "-lkayavm-opt" else "-lkayavm"
 	nocheck = if (nortchecks opts) then "-DNOCHECK " else ""
-        doWriteIface Module ifile ok = writeIface ifile ok
-	doWriteIface _ _ _ = return ()
-	doWriteXMLDocs True dfile ok inf = writeXMLDocs dfile ok inf
-	doWriteXMLDocs _ _ _ _ = return ()
+        doWriteIface Module inls ifile ok = writeIface inls ifile ok
+	doWriteIface _ _ _ _ = return ()
+	doWriteXMLDocs True dfile ok inf mdocstr = writeXMLDocs dfile ok inf mdocstr
+	doWriteXMLDocs _ _ _ _ _ = return ()
 	doWriteHTMLDocs True dfile ok inf = writeHTMLDocs dfile ok inf
 	doWriteHTMLDocs _ _ _ _ = return ()
-	addfnmap Webapp ok comp dk = comp++((mkfnmap ok):dk)
-	addfnmap Webprog ok comp dk = comp++((mkfnmap ok):dk)
-	addfnmap Program ok comp dk = comp++((mkfnmap ok):dk)
-	addfnmap Shebang ok comp dk = comp++((mkfnmap ok):dk)
+--	addfnmap Webapp ok comp dk = comp++((mkfnmap ok):dk)
+--	addfnmap Webprog ok comp dk = comp++((mkfnmap ok):dk)
+	addfnmap (Program _) ok comp dk = comp++((mkfnmap ok):(mkStartup ok):dk)
+	addfnmap Shebang ok comp dk = comp++((mkfnmap ok):(mkStartup ok):dk)
 	addfnmap Module ok comp dk = comp
 	dump True tac = putStr $ showtac tac
 	dump False tac = return ()
@@ -171,9 +191,6 @@ comp' t root libdirs opts ctxt ok extra mod = do
                       " -L"++(stripSlash x)++" "++incl xs
 
 
-hash [] = 0
-hash (x:xs) = 131*(hash xs)+(fromEnum x)
-
 showIfTrue :: Bool -> String -> IO ()
 showIfTrue True str = putStrLn str
 showIfTrue False _ = return ()
