@@ -6,13 +6,17 @@
     Public Licence. See COPYING for licence.
 -}
 
-module Language(module AbsSyntax, module Language) where
+module Language(module AbsSyntax, 
+                module Context,
+                module Language) where
 
 ----- Useful gadgets on the abstract syntax; also reexports syntax types.
 
 import List
 import Debug.Trace
 import AbsSyntax
+import Context
+import Options
 
 -- Merge somehow with 'Error' and tidy up name lookup functions.
 data Lookup a = Got a
@@ -33,6 +37,7 @@ instance Show a => Show (Lookup a) where
 
 -- Lookup a name, bearing in mind namespaces.
 -- Returns all possibilities, in the current module and others.
+{-
 lookupname :: Name -> -- current module
               Name -> -- name to lookup (possibly decorated)
 	      [(Name,a)] -> [(Name,a)]
@@ -55,7 +60,7 @@ lookupname mod n gam = checkCurrent $ lu n (decorated n) gam [] where
     decorated _ = False
     nameMatches n (NS _ a) = nameMatches n a
     nameMatches n x = n == x
-
+-}
 
 
 -- Lookup in the context (ignores namespace if no ambiguity, returns
@@ -64,20 +69,41 @@ lookupname mod n gam = checkCurrent $ lu n (decorated n) gam [] where
 ctxtlookup :: Monad m => Name -> -- Current module
 	                 Name -> Context -> 
                          Maybe Type -> -- type information to help disambiguate
+                         Options -> -- compiler options
                          m (Name, Type)
-ctxtlookup mod n ctx ty
+ctxtlookup mod n ctx ty copts
     = do let opts = nub (lookupname mod n ctx)
 	 let pub = filter accessible opts
 	 let priv = opts \\ pub
-	 returnName (tymatch ty (nubnames pub)) priv
+         let pubs = (tymatch ty (nubnames pub))
+         let depr = checkDepr pubs
+	 returnName pubs priv depr
 
-  where returnName [(x,(ty,opts))] _ = return (x,ty)
-	returnName [] [] = fail $ "Unknown name " ++ showuser n
-	returnName [] priv = fail $ "Can't use private name: " ++ 
+  where returnName _ _ True = case ty of 
+                               (Just jty) -> fail $ "Can't use deprecated function " ++ showuser n ++ "(" ++ showargs jty ++ ")"
+                               Nothing -> fail $ "Can't use deprecated function " ++ showuser n
+        returnName [(x,(ty,opts))] _ False = return (x,ty)
+        returnName [] [] False = fail $ "Unknown name " ++ showuser n
+	returnName [] priv False = fail $ "Can't use private name: " ++ 
 			         showuser n ++ " (" ++ showthings priv ++ ")"
-	returnName pub _ = fail $ "Ambiguous name: " ++ showuser n ++ 
+	returnName pub _ False = fail $ "Ambiguous name: " ++ showuser n ++ 
 			          " (Could be " ++ 
 				  showthings pub ++ ")"
+        
+        checkDepr [] = False
+-- only need to check first element
+        checkDepr ((x,(ty,opts)):xs) = checkDepr' (elem DeprecatedFn opts)
+-- something else will error first here
+--        checkDepr _ = False
+
+        checkDepr' False = False
+        checkDepr' True = useDepr (elem DeprFail copts) (elem DeprWarn copts)
+
+        useDepr False False = False
+        useDepr True _ = True
+-- FIXME: print a warning message below but succeed
+        useDepr False True = False
+
 	nubnames [] = []
 	nubnames (f@(x,(ty,opts)):xs) | (x,ty) `elem` (map getpair xs) = nubnames xs
 				      | otherwise = f:(nubnames xs)
@@ -88,7 +114,8 @@ ctxtlookup mod n ctx ty
                            x -> x
 
         matchesty (Just (Fn _ args _)) (x, ((Fn _ args2 _), opts))
-            = and $ zipWith matchArgs args args2
+            | (length args) <= (length args2) = and $ zipWith matchArgs args args2
+            | otherwise = False
         matchesty _ _ = True
 
         matchArgs x y | x == y = True
@@ -110,19 +137,10 @@ ctxtlookup mod n ctx ty
         showargs (Fn _ args _) = showlist args
         showargs _ = ""
 
-showlist [] = ""
-showlist [x] = show x
-showlist (x:xs) = show x ++ ", " ++ showlist xs
-
 showStrs [] = ""
 showStrs [x] = x
 showStrs (x:xs) = x ++ ", " ++ showStrs xs
 
-ctxtdump :: Context -> String
-ctxtdump [] = ""
-ctxtdump ((n,(ty,_)):xs) = showuser n ++ " :: " ++ show ty ++ "\n" ++ ctxtdump xs
-
-
 ------------ Gadgets -------------
 
 -- Return whether one type is "smaller" than another
@@ -145,9 +163,10 @@ biggert x y = x
 
 mangling :: Type -> String
 mangling t = "_" ++ mangling' t
-mangling' (Fn _ args _) = concat (map mangling' args)
+mangling' (Fn _ args _) = "F" ++ concat (map mangling' args)
 mangling' (Array arg) = "a" ++ mangling' arg
 mangling' (User n args) = show n ++ concat (map mangling' args)
+mangling' (TyVar _) = "P"
 mangling' (Prim Number) = "i"
 mangling' (Prim Character) = "c"
 mangling' (Prim Boolean) = "b"
@@ -205,6 +224,9 @@ numargs _ = 0
 -- Give distinct type variables fresh names, so that independent variables
 -- continue to be independent.
 
+-- Okay, so this really ought to be called 'generalise', like in Algorithm W
+-- for Hindley Milner inference. I couldn't remember the name at the time...
+
 fudgevars :: Type -> Int -> (Type, Int)
 fudgevars t next = let (vsmap,next') = newnames next (getUserVars t) in
 		       (alpha vsmap t, next')
@@ -260,9 +282,19 @@ typelookup mod t ti = returnName (nubnames (lookupname mod t ti))
 	 showthings [(x,_)] = showuser x
 	 showthings ((x,_):xs) = showuser x ++ ", " ++ showthings xs
 
-	 nubnames [] = []
-	 nubnames (f@(x,_):xs) | x `elem` (map fst xs) = nubnames xs
-			       | otherwise = f:(nubnames xs)
+nubnames [] = []
+nubnames (f@(x,_):xs) | x `elem` (map fst xs) = nubnames xs
+	              | otherwise = f:(nubnames xs)
+
+-- Lookup in the exception context (ignores namespace if no ambiguity, returns
+-- fully qualified type name)
+exceptlookup :: Name -> -- Current module
+	        Name -> EContext -> Lookup (Name, [Type])
+exceptlookup mod e ei = returnName (nubnames (lookupname mod e ei))
+   where returnName [x] = Got x
+	 returnName [] = UnknownName e
+--fail $ "Unknown type " ++ show t
+	 returnName xs = Ambiguous e (map fst xs)
 
 -- Type normalisation; expand synonyms.
 
@@ -504,3 +536,128 @@ foldsubexpr f com def expr = app expr
 	altapp ((ConstAlt pt c e):as) 
 	    = (f e) `com` (altapp as)
 
+locsUsed :: Expr n -> [Int]
+locsUsed (Loc i) = [i]
+locsUsed (Lambda _ _ e) = locsUsed e
+locsUsed (Closure _ _ e) = locsUsed e
+locsUsed (Bind _ _ e1 e2) = locsUsed e1 ++ locsUsed e2
+locsUsed (Declare _ _ _ _ e) = locsUsed e
+locsUsed (Return e) = locsUsed e
+locsUsed (Assign a e) = alocsUsed a ++ locsUsed e
+locsUsed (AssignOp _ a e) = alocsUsed a ++ locsUsed e
+locsUsed (AssignApp a e) = alocsUsed a ++ locsUsed e
+locsUsed (Seq x y) = locsUsed x ++ locsUsed y
+locsUsed (Apply f as) = locsUsed f ++ concat (map locsUsed as)
+locsUsed (Partial f as _) = locsUsed f ++ concat (map locsUsed as)
+locsUsed (Foreign _ _  as) = concat (map locsUsed (map fst as))
+locsUsed (While x y) = locsUsed x ++ locsUsed y
+locsUsed (DoWhile x y) = locsUsed x ++ locsUsed y
+locsUsed (For _ _ _ a x y) = alocsUsed a ++ locsUsed x ++ locsUsed y
+locsUsed (TryCatch x y z w) = locsUsed x ++ locsUsed y ++
+                              locsUsed z ++ locsUsed w
+locsUsed (NewTryCatch x c) = locsUsed x ++ concat (map clocsUsed c)
+locsUsed (Throw x) = locsUsed x
+locsUsed (Except x y) = locsUsed x ++ locsUsed y
+locsUsed (NewExcept xs) = concat (map locsUsed xs)
+locsUsed (Infix _ x y) = locsUsed x ++ locsUsed y
+locsUsed (RealInfix _ x y) = locsUsed x ++ locsUsed y
+locsUsed (CmpExcept _ x y) = locsUsed x ++ locsUsed y
+locsUsed (CmpStr _ x y) = locsUsed x ++ locsUsed y
+locsUsed (Append x y) = locsUsed x ++ locsUsed y
+locsUsed (AppendChain xs) = concat (map locsUsed xs)
+locsUsed (Unary _ x) = locsUsed x
+locsUsed (RealUnary _ x) = locsUsed x
+locsUsed (Coerce _ _ x) = locsUsed x
+locsUsed (Case e cs) = locsUsed e ++ concat (map caseLocsUsed cs)
+locsUsed (If t x y) = locsUsed t ++ locsUsed x ++ locsUsed y
+locsUsed (Index x y) = locsUsed x ++ locsUsed y
+locsUsed (Field e _ _ _) = locsUsed e
+locsUsed (ArrayInit xs) = concat (map locsUsed xs)
+locsUsed (Annotation _ e) = locsUsed e
+locsUsed _ = []
+
+alocsUsed (AIndex a e) = alocsUsed a ++ locsUsed e
+alocsUsed (AField a _ _ _) = alocsUsed a
+alocsUsed _ = []
+
+clocsUsed (Catch (Right e) h) = locsUsed e ++ locsUsed h
+clocsUsed (Catch (Left (n,es)) h) = concat (map locsUsed es) ++ locsUsed h
+
+caseLocsUsed (Default e) = locsUsed e
+caseLocsUsed (Alt _ _ es e) = concat (map locsUsed es) ++ locsUsed e
+caseLocsUsed (ConstAlt _ _ e) = locsUsed e
+
+-- get a list of all variables which are modified (i.e. assigned to,
+-- or passed to a function as a var argument) in a block
+
+modified expr = modVar expr -- foldsubexpr modVar (++) [] expr
+
+modVar :: Expr n -> [Int]
+modVar (Lambda _ _ e) = modVar e
+modVar (Closure _ _ e) = modVar e
+modVar (Bind _ _ v e) = modVar v ++ modVar e
+modVar (Declare _ _ _ _ e) = modVar e
+modVar (Assign lval ex) = inLval lval ++ modVar ex
+modVar (AssignOp _ lval ex) = inLval lval ++ modVar ex
+modVar (AssignApp lval ex) = inLval lval ++ modVar ex
+modVar (Seq x y) = modVar x ++ modVar y
+modVar (Annotation a e) = modVar e
+modVar (For _ _ _ l a b) = inLval l ++ modVar b ++ modVar a
+modVar (While t e) = modVar t ++ modVar e
+modVar (DoWhile e t) = modVar t ++ modVar e
+modVar (NewExcept es) = concat (map modVar es)
+modVar (Except e t) = modVar t ++ modVar e
+modVar (Infix _ x y) = modVar x ++ modVar y
+modVar (RealInfix _ x y) = modVar x ++ modVar y
+modVar (Unary _ x) = modVar x
+modVar (RealUnary _ x) = modVar x
+modVar (Coerce _ _ e) = modVar e
+modVar (Index x y) = modVar x ++ modVar y
+modVar (Field x _ _ _) = modVar x
+modVar (ArrayInit es) = concat (map modVar es)
+modVar (NewTryCatch e cs) = modVar e ++ concat (map cmodVar cs)
+-- This ought to check if the arg is a var arg.
+modVar (Apply _ args) = concat (map locsUsed args)
+-- May be over conservative, but unlikely to happen much in practice.
+modVar (Partial _ args _) = concat (map locsUsed args)
+modVar (Foreign _ _ args) = concat (map locsUsed (map fst args))
+modVar (Case e alts) = modVar e ++ concat (map inAlt alts)
+modVar (If x t e) = modVar x ++ modVar t ++ modVar e
+modVar (Return e) = modVar e
+modVar (Throw e) = modVar e
+modVar x = []
+
+isArg (Loc i) = [i]
+isArg _ = []
+
+inAlt (Alt _ _ args e) = modVar e ++ concat (map isArg args)
+inAlt (ConstAlt _ _ e) = modVar e
+inAlt (Default e) = modVar e
+
+inLval (AName i) = [i]
+inLval (AIndex lval _) = inLval lval
+inLval (AField lval _ _ _) = inLval lval
+inLval _ = []
+
+cmodVar (Catch (Right nm) e) = modVar nm ++ modVar e
+cmodVar (Catch (Left (_,es)) e) = concat (map modVar es) ++ modVar e
+
+-- Functions which the compiler assumes to exist
+
+eqfun = NS (UN "Builtins") (UN "equal")
+eqmangle = mangling (Fn [] [TyVar (UN "a"), TyVar (UN "a")] (Prim Boolean))
+
+sizefn = NS (UN "Builtins") (UN "size")
+sizemangle = mangling (Fn [] [Array (TyVar (UN "a"))] (Prim Number))
+
+missingCase = NS (UN "Builtins") (UN "Missing_Case")
+missingCaseMangling = mangling (Fn [] [] (Prim Exception));
+
+exitfun = (NS (UN "Builtins") (UN "exit"))
+exitmangle = mangling (Fn [] [(Prim Number)] (Prim Void))
+
+putstrlnfun = (NS (UN "Prelude") (UN "putStrLn"))
+putstrlnmangle = mangling (Fn [] [(Prim StringType)] (Prim Void))
+
+backtracefun = (NS (UN "Builtins") (UN "exceptionBacktrace"))
+backtracemangle = mangling (Fn [] [(Prim Exception)] (Prim Void))
\ No newline at end of file
