@@ -34,6 +34,9 @@ getContent = \s fn ld mod l -> Success s
 getModuleName :: P Name
 getModuleName = \s fn ld mod l -> Success mod
 
+setLineNo :: LineNumber -> P a -> P a
+setLineNo i p = \s fn ld mod l -> p s fn ld mod i
+
 thenP :: P a -> (a -> P b) -> P b
 m `thenP` k = \s fn ld mod l ->
    case m s fn ld mod l of 
@@ -128,6 +131,9 @@ data Token = TokenInt Int
 	   | TokenFinally
 	   | TokenThrow
 	   | TokenException
+	   | TokenNewException
+	   | TokenExtException
+	   | TokenNewTry
 	   | TokenTo
 	   | TokenIn
 	   | TokenCase
@@ -147,13 +153,20 @@ data Token = TokenInt Int
 	   | TokenWebprog
 	   | TokenShebang
 	   | TokenExtern
+	   | TokenExternInline
+	   | TokenCycleExtern
+           | TokenFunMap
 	   | TokenLifted
 	   | TokenExtData
 	   | TokenImported
 	   | TokenDocstring
+           | TokenDeprecated
 	   | TokenLink
+	   | TokenLine
+	   | TokenFile
            | TokenTest
 	   | TokenVM
+           | TokenStartup
 	   | TokenGlobal
 	   | TokenPublic
 	   | TokenPrivate
@@ -228,6 +241,7 @@ lexer cont (',':cs) = cont TokenComma cs
 lexer cont ('.':'.':cs) = cont TokenDots cs
 lexer cont ('.':cs) = cont TokenDot cs
 lexer cont ('`':cs) = cont TokenBacktick cs
+lexer cont ('\\':cs) = cont TokenLambda cs
 lexer cont ('#':'!':cs) = cont TokenShebang (stripToNL cs)
    where stripToNL ('\n':cs) = cs
 	 stripToNL (x:cs) = stripToNL cs
@@ -299,11 +313,11 @@ lexVar cont cs =
       ("for",rest) -> cont TokenFor rest
       ("break",rest) -> cont TokenBreak rest
       ("pass",rest) -> cont TokenPass rest
-      ("try",rest) -> cont TokenTry rest
+      ("try",rest) -> cont TokenNewTry rest
       ("throw",rest) -> cont TokenThrow rest
       ("catch",rest) -> cont TokenCatch rest
       ("finally",rest) -> cont TokenFinally rest
-      ("Exception",rest) -> cont TokenException rest
+      ("Exception",rest) -> cont TokenNewException rest
 --      ("to",rest) -> cont TokenTo rest
       ("in",rest) -> cont TokenIn rest
       ("case",rest) -> cont TokenCase rest
@@ -321,21 +335,24 @@ lexVar cont cs =
 --      ("CINCLUDE",rest) -> cont TokenCInclude rest
       ("include",rest) -> cont TokenInclude rest
       ("import",rest) -> cont TokenImport rest
-      ("program",rest) -> cont TokenProgram rest
+--      ("program",rest) -> cont TokenProgram rest
       ("module",rest) -> cont TokenModule rest
-      ("webapp",rest) -> cont TokenWebapp rest
-      ("webprog",rest) -> cont TokenWebprog rest
+--      ("webapp",rest) -> cont TokenWebapp rest
+--      ("webprog",rest) -> cont TokenWebprog rest
 --      ("fnid",rest) -> cont TokenFNid rest
       ("globals",rest) -> cont TokenGlobal rest
       ("lambda",rest) -> cont TokenLambda rest
       ("public",rest) -> cont TokenPublic rest
       ("private",rest) -> cont TokenPrivate rest
       ("pure",rest) -> cont TokenPure rest
+      ("extern",rest) -> cont TokenCycleExtern rest
       (var,rest)   -> cont (mkname var) rest
 
 lexSpecial cont cs =
     case span isAllowed cs of
       ("extern",rest) -> cont TokenExtern rest
+      ("extinline",rest) -> cont TokenExternInline rest
+      ("fnmap",rest) -> cont TokenFunMap rest
       ("lifted",rest) -> cont TokenLifted rest
       ("data",rest) -> cont TokenExtData rest
       ("datacon",rest) -> cont TokenDatacon rest
@@ -343,12 +360,41 @@ lexSpecial cont cs =
       ("type",rest) -> cont TokenExtType rest
       ("doc",rest) -> cont TokenDocstring rest
       ("imported",rest) -> cont TokenImported rest
+      ("deprecated",rest) -> cont TokenDeprecated rest
       ("include",rest) -> cont TokenCInclude rest
       ("link",rest) -> cont TokenLink rest
+      ("line",rest) -> setLine cont rest
+      ("file",rest) -> setFile cont rest
       ("test",rest) -> cont TokenTest rest
       ("VM",rest) -> cont TokenVM rest
+      ("startup",rest) -> cont TokenStartup rest
+      ("Exception",rest) -> cont TokenException rest
+      ("except",rest) -> cont TokenExtException rest
+      ("try",rest) -> cont TokenTry rest
       (thing,rest) -> cont TokenMod (thing++rest)
 
+setLine cont (x:xs) 
+    | isSpace x = setLine cont xs
+    | isDigit x = case span isDigit (x:xs) of
+                    (num, rest) -> 
+                        \fn ld mod line -> lexer cont rest fn ld mod (read num)
+    | otherwise = \fn ld mod line ->
+                     failP (fn++":"++show line++":Invalid %line directive")
+		       xs fn ld mod line
+
+
+setFile cont (x:xs) 
+    | isSpace x = setFile cont xs
+    | x == '"'  = case getstr xs of
+                    (Just (str, rest, nls)) -> 
+                      \fn ld mod line -> lexer cont rest str ld mod (line+nls)
+                    Nothing -> \fn ld mod line ->
+                     failP (fn++":"++show line++":Invalid %file directive")
+		       xs fn ld mod line
+    | otherwise = \fn ld mod line ->
+                     failP (fn++":"++show line++":Invalid %file directive")
+		       xs fn ld mod line
+
 mkname :: String -> Token
 mkname c = TokenName (UN c)
 
