@@ -15,12 +15,19 @@ import List
 import Lib
 import Options
 import Portability
+import Inliner
 
 importVersion :: Int
-importVersion = 2
+importVersion = 3
 
-writeIface :: FilePath -> Program -> IO ()
-writeIface fn prog
+-- If something is inlinable and very small (e.g. just a constant)
+-- write out its definition. We pass in the inlinable definitions, but
+-- this is currently unused, we just work it out ourselves here since
+-- it's not quite the same functions which can be exported inlinable
+-- (complications with globals and privates)
+
+writeIface :: Inlinable -> FilePath -> Program -> IO ()
+writeIface inls fn prog
   = do let str = mkIface prog
        writeFile fn str
 
@@ -30,10 +37,23 @@ mkIface (CInclude str:xs) = mkIface xs
 mkIface (Imported str:xs) = "%imported "++show str++";\n"++mkIface xs
 mkIface (Linker str:xs) = "%link "++show str++";\n"++mkIface xs
 mkIface ((FunBind (f,l,n,ty,fopts,Unbound) _ _):xs) 
-    | elem Export fopts = mkExt n ty fopts ++ "\n" ++ mkIface xs
-    | otherwise = mkExt n ty (delete Public fopts) ++ "\n" ++ mkIface xs
-mkIface ((FunBind (f,l,n,ty,fopts,Defined _) _ _):xs)
-   | not (Generated `elem` fopts) = mkExt n ty fopts ++ "\n" ++ mkIface xs
+    | elem Repeatable fopts = mkIface xs
+    | elem Export fopts = mkExt n ty fopts Nothing ++ "\n" ++ mkIface xs
+    | otherwise = mkExt n ty (delete Public fopts) Nothing ++ "\n" ++ mkIface xs
+--    | otherwise = "%fnmap \"" ++ show n ++ mangling ty ++ "\""
+--                     ++ "\n" ++ mkIface xs
+mkIface ((FunBind (f,l,n,ty,fopts,ExtInlinable def) _ _):xs) 
+    | elem Repeatable fopts = mkIface xs
+    | elem Export fopts = mkExt n ty fopts (showInl def) ++ "\n" ++ mkIface xs
+    | otherwise = mkExt n ty (delete Public fopts) (showInl def) 
+                     ++ "\n" ++ mkIface xs
+--    | otherwise = "%fnmap \"" ++ show n ++ mangling ty ++ "\"" ++ "\n" ++ mkIface xs
+mkIface ((FunBind (f,l,n,ty,fopts,ExceptionFn nm ar _) _ _):xs) 
+    -- always export exception declarations
+    = mkExcept nm ty ++ "\n" ++ mkIface xs
+mkIface ((FunBind (f,l,n,ty,fopts,Defined def) _ _):xs)
+   | not (Generated `elem` fopts) = mkExt n ty fopts (showInl def) 
+                                       ++ "\n" ++ mkIface xs
    | otherwise = mkExtCName n ty ++ "\n" ++ 
                  mkIface xs -- write the C name, to keep FunMap correct
 --mkIface (FunBind (n,ty,DataCon i ar):xs) = 
@@ -50,31 +70,53 @@ mkIface ((TySyn (f,l,n,ps,ty,True)):xs) =
    mkTySyn n ps ty ++ "\n" ++ mkIface xs
 mkIface (_:xs) = mkIface xs
 
+mkExcept nm (Fn _ args ret) = 
+    "%except " ++ showuser nm ++ "(" ++ showargs args ++ ");"
+  where showargs [] = ""
+	showargs [x] = show x
+	showargs (x:xs) = show x ++ "," ++ showargs xs
+
 mkExtCName n ty = "%lifted \"" ++ show n ++ mangling ty ++ "\""
 
-mkExt :: Name -> Type -> [FOpt] -> String
-mkExt n (Fn defaults args ret) fopts 
-   = "%extern " ++show importVersion ++ " "++ writefopts fopts ++ show ret ++ " " ++ showuser n ++ 
-     (if elem NoArgs fopts then ";" else "(" ++ showargs defaults args ++ ");")
-  where showargs [] [] = ""
-	showargs [d] [x] = show x ++ showdef d
-	showargs (d:ds) (x:xs) = show x ++ showdef d ++ "," ++ showargs ds xs
+mkExt :: Name -> Type -> [FOpt] -> Maybe (String, [Name]) -> String
+mkExt n (Fn defaults args ret) fopts def
+   = extTok def ++ show importVersion ++ " "++ writefopts fopts ++ show ret ++ " " ++ showuser n ++ 
+     (if elem NoArgs fopts then "" else "(" ++ showargs defaults args (getArgs def) ++ ")")
+     ++ extShowDef def
+  where showargs [] [] _ = ""
+	showargs [d] [x] [a] = show x ++ " " ++ showuser a ++ showdef d
+	showargs [d] [x] [] = show x ++ showdef d
+	showargs (d:ds) (x:xs) [] 
+            = show x ++ showdef d ++ "," ++ showargs ds xs []
+	showargs (d:ds) (x:xs) (a:as) 
+            = show x ++ " " ++ showuser a ++ showdef d ++ "," 
+              ++ showargs ds xs as
 
         showdef Nothing = "";
 	showdef (Just a) = " = " ++ showexp a -- FIXME: Do it properly
+        getArgs (Just (_,as)) = as
+        getArgs _ = []
 	writefopts (Public:xs) = "public " ++ writefopts xs
 	writefopts (Pure:xs) = "pure " ++ writefopts xs
+	writefopts (StartupFn:xs) = "%startup " ++ writefopts xs
+	writefopts (DeprecatedFn:xs) = "%deprecated " ++ writefopts xs
 	writefopts (_:xs) = writefopts xs
 	writefopts [] = ""
-mkExt n t fopts = "%extern " ++ show importVersion ++ " " ++ 
+mkExt n t fopts def = extTok def ++ show importVersion ++ " " ++ 
                   writefopts fopts ++ 
 		  show t ++ " " ++ showuser n ++ 
-     (if elem NoArgs fopts then ";" else "();")
+     (if elem NoArgs fopts then "" else "()") ++
+     extShowDef def
    where writefopts (Public:xs) = "public " ++ writefopts xs
 	 writefopts (Pure:xs) = "pure " ++ writefopts xs
 	 writefopts (_:xs) = writefopts xs
 	 writefopts [] = ""
 
+extTok Nothing = "%extern "
+extTok (Just _) = "%extinline "
+extShowDef Nothing = ";"
+extShowDef (Just (def, _)) = "{ " ++ def ++ " };"
+
 
 mkData :: Name -> [Type] -> [DOpt] -> [ConDecl] -> String
 mkData n args opts cons = "%data " ++ show importVersion ++ " " ++ 
