@@ -1,26 +1,48 @@
 module Optimise where
 
 import Language
+import Inliner
+import Propagate
+
 import Debug.Trace
 
 -- An optimisation is a transformation from an expression to an 
 -- optimised expression
 data Optimisation n = Opt String (Expr n -> Expr n)
 
+
 -- Naturally then, optimisations can be composed.
 optComp (Opt fn f) (Opt gn g) = Opt (gn++ " -> "++fn) (f.g)
 
 -- And we have an identity optimisation
 optID = Opt "" id
 
+-- Top level optimisation loop
+-- Optimisation levels are currently:
+--    0: Just parse time constant folding
+--    1: Inlining, propagation, for loop collapsing, append collapsing,
+--       remaining constant folds
+--       Do some of them twice to catch anything which previous steps may 
+--       expose.
+
+dontimes 0 f x = x
+dontimes n f x = f (dontimes (n-1) f x)
+
+runOpts :: Int -> Program -> Program
+runOpts 0 p = p
+runOpts 1 p 
+    = dontimes 2 (copyPropagate.(optAll basicOpts).inliner.copyPropagate.(optAll basicOpts)) p
+runOpts n p = runOpts (n-1) p
+
 basicOpts :: Optimisation Name
 basicOpts = forOpt `optComp`
             appendOpt `optComp`
+            constantFold `optComp`
 	    optID
 
-allOpts = [forOpt,appendOpt]
+allOpts = [forOpt,appendOpt,constantFold]
 
-allOptNames = ["forloop","append"]
+allOptNames = ["forloop","append","constant"]
 
 useOpts :: Monad m => [String] -> m (Optimisation Name)
 useOpts [] = return optID
@@ -31,13 +53,41 @@ useOpts (x:xs) = do rest <- useOpts xs
 	 findOpt x ((opt@(Opt n _)):os) | n == x = return opt
 					| otherwise = findOpt x os
 
-
-
 {-testOpt = let src = Infix Plus (GConst (Num 4)) (Metavar "foo" 0 0)
 	      dest = Infix Plus (Metavar "foo" 0 0) (Metavar "foo" 0 0) in
 	      genericOpt "test" src dest
 -}
 
+-- Although we do this at parse time, we might find a few more arising
+-- from other optimisations, especially inlining.
+constantFold = Opt "constant" cfold'
+  where cfold' e@(Infix op (GConst (Num x)) (GConst (Num y)))
+            = case foldint op x y of
+                 Nothing -> e
+                 Just v -> GConst v
+        cfold' e@(RealInfix op (GConst (Re x)) (GConst (Re y)))
+            = case foldreal op x y of
+                 Nothing -> e
+                 Just v -> GConst v
+        cfold' e@(Unary op (GConst (Num x)))
+            = case foldunint op x of
+                 Nothing -> e
+                 Just v -> GConst v
+        cfold' e@(RealUnary op (GConst (Re x)))
+            = case foldunreal op x of
+                 Nothing -> e
+                 Just v -> GConst v
+        cfold' e@(If (GConst (Bo True)) t f) = t
+        cfold' e@(If (GConst (Bo False)) t f) = f
+        cfold' (Coerce (Prim Number) (Prim RealNum) (GConst (Num x)))
+            = (GConst (Re (fromIntegral x)))
+        cfold' c@(Coerce t1 t2 e) 
+            = let e' = cfold' e in
+                  if (e/=e')
+                     then (cfold' (Coerce t1 t2 e'))
+                     else c
+        cfold' x = x
+
 appendOpt = Opt "append" appOpt'
   where appOpt' (Assign lval (AppendChain [apped, v]))
                    | same lval apped = AssignApp lval v
