@@ -13,7 +13,8 @@ import Lib
 
 data Option = NoPrelude | DumpTAC | DumpTree | DumpRaw | KeepC | ShowGCC
 	    | DumpEqns | DumpCG | DumpDeps | Libdir String | XMLDocs | HTMLDocs
-	    | SeedKey String | ForceBuild | NoChase | Quiet
+	    | SeedKey String | ForceBuild | NoChase | Quiet 
+            | DeprWarn | DeprFail | StaticExecutable
 	    | DynLink String | NoOpts | Profile | NoRTChecks | NoEnvLibs
   deriving (Show, Eq)
 
@@ -27,10 +28,13 @@ dumpraw opts = elem DumpRaw opts
 dumpeqns opts = elem DumpEqns opts
 dumpcg opts = elem DumpCG opts
 dumpdeps opts = elem DumpDeps opts
+static opts = elem StaticExecutable opts
 keepc opts = elem KeepC opts
 showgcc opts = elem ShowGCC opts
 xmldocs opts = elem XMLDocs opts
 htmldocs opts = elem HTMLDocs opts
+deprwarn opts = elem DeprWarn opts
+deprfail opts = elem DeprFail opts
 forcebuild opts = elem ForceBuild opts || elem NoChase opts
 nochase opts = elem NoChase opts
 quiet opts = elem Quiet opts || elem NoChase opts
@@ -80,6 +84,8 @@ getopts ("-dumpcg":xs) = do (opts,rest) <- getopts xs
 			    return (DumpCG:opts,rest)
 getopts ("-dumpdeps":xs) = do (opts,rest) <- getopts xs
 			      return (DumpDeps:opts,rest)
+getopts ("-static":xs) = do (opts,rest) <- getopts xs
+			    return (StaticExecutable:opts,rest)
 getopts ("-keepc":xs) = do (opts,rest) <- getopts xs
 			   return (KeepC:opts,rest)
 getopts ("-showgcc":xs) = do (opts,rest) <- getopts xs
@@ -96,6 +102,10 @@ getopts ("-xmldocs":xs) = do (opts,rest) <- getopts xs
 			     return (XMLDocs:opts,rest)
 getopts ("-htmldocs":xs) = do (opts,rest) <- getopts xs
 			      return (HTMLDocs:opts,rest)
+getopts ("-deprwarn":xs) = do (opts,rest) <- getopts xs
+			      return (DeprWarn:opts,rest)
+getopts ("-deprfail":xs) = do (opts,rest) <- getopts xs
+			      return (DeprFail:opts,rest)
 getopts ("-force":xs) = do (opts,rest) <- getopts xs
 			   return (ForceBuild:opts,rest)
 getopts ("-nochase":xs) = do (opts,rest) <- getopts xs
