@@ -19,9 +19,10 @@ import List
 import KeyGen
 
 data CompileResult = RawCode String  -- Raw C, eg for #includes
-		   | ByteCode (Name,Type,[TAC]) -- Function name and bytecode 
+		   | ByteCode (Name,Type,Code) -- Function name and bytecode 
 		   | GlobCode Int -- Global variable
 		   | ExternDef (Name, Type)
+                   | ExcCode Name Bool
    deriving Show
 
 type Var = (Name, Bool) -- Bool now unused, used to mean whether it went on the stack or heap...
@@ -49,16 +50,19 @@ data TAC = DECLARE Var
 	 | RTMPSET Tmp Double
 	 | CALL Var
 	 | CALLNAME Mangled
+         | FASTCALL Mangled [Var]
 	 | CALLTOP
 	 | TAILCALL Var
 	 | TAILCALLNAME Mangled
 	 | TAILCALLTOP
 	 | CLOSURE Mangled Int Int
 	 | CLOSURELOC Var Int 
+	 | CLOSURETOP Int 
 	 | FOREIGNCALL String String Type [Type] -- name, library, type, args
 	 | MKCON Int Int -- Create a constructor, tag and arity
 	 | MKCONZERO Int -- Create a constructor, tag with arity zero
 	 | MKEXCEPT -- Create an exception from top two stack items
+	 | MKNEWEXCEPT Name Int -- Create an exception with name and arity
 	 | EQEXCEPT -- Check two exceptions for equality
 	 | NEEXCEPT -- Check two exceptions for inequality
 	 | EQSTRING -- Check two strings for equality
@@ -69,6 +73,7 @@ data TAC = DECLARE Var
          | JNESTRING Name -- jump if two string not equal
          | JEQSTRINGW Name String -- jump if string equal to stack top
          | JNESTRINGW Name String -- jump if string not equal to stack top
+         | JEXNE Name Name -- jump if top exception not equal to stack top
 	 | GETFUN Var Var
 	 | GETVAL Tmp
 	 | GETRVAL Tmp
@@ -78,6 +83,7 @@ data TAC = DECLARE Var
          | PUSHGETINDEX PushItem
          | PUSHTOINDEX PushItem
 	 | PROJARG Int Int -- Project first arg, check tag is second arg.
+	 | EXPROJARG Var Int -- Project into first arg from second arg of exception
 	 | INFIX Tmp Op Tmp Tmp
          | ADDINPLACE PushItem Int
          | SUBINPLACE PushItem Int
@@ -175,9 +181,9 @@ tmpval i = UN ("tv" ++ show i)
 
 type Local = Int
 
-comp :: Monad m => Program -> Expr Name -> Bool -> Name -> m [TAC]
-comp ds e pure mod = do code <- fcompile ds e pure mod
-			return $ peephole $ decls $ code
+comp :: Monad m => Program -> Expr Name -> Bool -> Name -> m Code
+comp ds e pure mod = do (Code pop args code) <- fcompile ds e pure mod
+			return (Code [] args (peephole (decls (pop++code))))
 
 -- Metadata for compilation of an expression
 data CompData = CD {
@@ -191,9 +197,17 @@ data CompData = CD {
                     varargs :: [Var]
 		   }
 
+data Code = Code {
+                  argPop :: [TAC], -- argument popping code
+                  argNames :: [(ArgType, Var)],
+                  mainCode :: [TAC] -- main body
+                 }
+   deriving Show
+
 -- Decorate names looked up from the context with their de bruijn index
 getVar :: [Var] -> Int -> Var
-getVar xs i = decorate (xs!!i) i
+getVar xs i | i<length xs = decorate (xs!!i) i
+            | otherwise = error $ (show xs) ++ show i
 
 decorate nm i = case nm of
                  (UN n, b) -> (UN (n++show i), b)
@@ -203,12 +217,32 @@ infixl 5 !>
 
 (!>) = getVar
 
-fcompile :: Monad m => Program -> Expr Name -> Bool -> Name -> m [TAC]
-fcompile ds exp pure mod = do
-   (code, tmps) <- fcompile' exp (CD mod [] 0 Nothing 0 cache ds []) 0
-   return code
- where cache | pure = Just 0
-	     | otherwise = Nothing
+fcompile :: Monad m => Program -> Expr Name -> Bool -> Name -> m Code
+fcompile ds (Annotation a e) pure mod =
+   do (Code pop args def) <- fcompile ds e pure mod
+      (acode, lab) <- annotcompile a def 0
+      return (Code pop args acode)
+fcompile ds (Lambda ivs args sc) pure mod 
+    = do let vs' = getvarargs 0 (zip ivs (map fst args))
+         (code,lab') <- fcompile' sc 
+                        (CD mod 
+                            (map (\x -> (fst x, False)) args)
+                            0 Nothing 0 (updatecache cache (length args)) ds vs') 
+                          0
+	 let acode = acompile ivs args
+	 let cachecode = case cache of
+			    Nothing -> []
+			    (Just x) -> [CHECKCACHE (length args)]
+	 return $ Code acode (aget ivs args) (cachecode ++ code ++ remember vs')
+  where updatecache Nothing l = Nothing
+	updatecache (Just _) l = Just l
+        getvarargs i [] = []
+        getvarargs i ((Var,x):xs) = (decorate (x, False) i):getvarargs (i+1) xs
+        getvarargs i (_:xs) = getvarargs (i+1) xs
+        cache | pure = Just 0
+              | otherwise = Nothing
+fcompile ds Noop pure mod = return $ Code [] [] []
+fcompile ds exp pure mod = fail $ "No function! Can't happen! " ++ show exp
 
 -- fcompile' needs to keep track of:
 --   local variable names ([Var])
@@ -240,10 +274,16 @@ fMkHeap (If c t e) cd lab
   where false = MN ("l",lab)
 	end = MN ("l",lab+1)
 	lab' = lab+2
+fMkHeap (Coerce t1 t2 v) cd lab = 
+   do (vcode,lab') <- fMkHeap v cd lab
+      cocompile t1 t2 vcode cd lab'
 fMkHeap (Case v alts) cd lab = 
    do (vcode,lab') <- fcompile' v cd lab
       (acode,lab'') <- altcompile True alts cd lab'
       return (vcode ++ acode, lab'')
+fMkHeap (Annotation a e) cd lab =
+   do (ecode, lab') <- fMkHeap e cd lab
+      annotcompile a ecode lab'
 fMkHeap x cd lab = fcompile' x cd lab
 
 -- Decide on a function based on whether the result goes on the heap or not
@@ -256,24 +296,6 @@ fcompile' (Global n mangling arity) _ lab
 fcompile' (Loc l) cd lab = return ([PUSH (VAR ((locs cd)!>l))], lab)
 fcompile' (GVar i) cd lab = return ([PUSHGLOBAL (show (modulename cd)) i], lab)
 fcompile' (GConst c) _ lab = ccompile_stack c lab
-fcompile' (Lambda ivs args sc) (CD mod vars tmp b tr cache ds vs) lab 
-    = do let vs' = vs ++ getvarargs 0 (zip ivs (map fst args))
-         (code,lab') <- fcompile' sc 
-                        (CD mod 
-                            (vars++(map (\x -> (fst x, False)) args))
-                            tmp b tr (updatecache cache (length args)) ds vs') 
-                          lab
-	 let acode = acompile ivs args
-	 let cachecode = case cache of
-			    Nothing -> []
-			    (Just x) -> [CHECKCACHE (length args)]
-	 return (acode ++ cachecode ++ code ++ remember vs', lab')
-  where updatecache Nothing l = Nothing
-	updatecache (Just _) l = Just l
-        getvarargs i [] = []
-        getvarargs i ((Var,x):xs) = (decorate (x, False) i):getvarargs (i+1) xs
-        getvarargs i (_:xs) = getvarargs (i+1) xs
-	
 fcompile' (Closure _ _ _) cd lab = fail "Encountered a closure. Something is probably broken."
 fcompile' (Bind n t v sc) (CD mod vars tmp b tr cache ds vs) lab 
     = do (vcode, lab') <- fcompile' v (CD mod vars tmp b tr cache ds vs) lab
@@ -355,14 +377,21 @@ fcompile' (Seq x y) cd lab
     = do (xcode, lab') <- fcompile' x cd lab 
 	 (ycode, lab'') <- fcompile' y cd lab'
 	 return (xcode ++ ycode, lab'')
+-- If we know the function name and it's fully applied, use the fast entry 
+-- point to save doing some unnecessary stack work
+{- but it's not quite right yet
+fcompile' (Apply (Global n m ar) as) cd lab
+    = do (argcode, args, lab') <- makeargs as cd lab
+         return (argcode ++ [FASTCALL (cname n "_fast_"++m) args], lab')
+-}
 fcompile' (Apply f as) cd lab 
     = do (pcode, lab') <- pushargs as cd lab
 	 (ccode, lab'') <- call f cd lab'
-	 return (pcode ++ ccode, lab')
+	 return (pcode ++ ccode, lab'')
 fcompile' (Partial f as i) cd lab 
     = do (pcode, lab') <- pushargsHeap as cd lab
 	 (ccode, lab'') <- closure f (length as) i cd lab'
-	 return (pcode ++ ccode, lab')
+	 return (pcode ++ ccode, lab'')
 fcompile' (Foreign ty f as) cd lab = foreigncomp ty f as cd lab
 fcompile' (While c sc) cd@(CD mod vars tmp b tr cache ds vs) lab
      = do let cd' = (CD mod vars tmp (Just end) tr cache ds vs)
@@ -388,27 +417,51 @@ fcompile' (For x uname y l ar e) (CD mod vars tmp b tr cache ds vs) lab
           (xcode, labs1) <- fcompile' (Assign (AName x) (GConst (Num 0))) cd' lab'
 	  (ycode, labs2) <- fcompile' (Assign (AName y) ar) cd' labs1
 -- Check if x equals the length of the array
-	  (testcode, labs3) <- fcompile' 
-	       (Infix Equal (Loc x) (Apply (Global sizefn "_a" 1) [(Loc y)])) cd' labs2 
-	  (acode, labs4) <- fcompile' (Assign l (Index (Loc y) (Loc x))) cd' labs3
+          let arrlen = Apply (Global sizefn sizemangle 1) [(Loc y)]
+          let arrmod = arrModified ar e
+          (lencode, labs3) <- 
+              if arrmod then return ([], labs2) else
+                 do (alcode, l') <- fcompile' arrlen cd' labs2
+                    return ([USETMP sizetmpvar] ++ alcode ++
+                            [PUSHSETTOP (VAR sizetmpvar)], l')
+          (testcode, labs4) <- -- trace (show (ar, e, arrmod)) $
+              if arrmod then fcompile' 
+	          (Infix Equal (Loc x) arrlen) cd' labs3
+                else 
+                  do return ([PUSH (VAR (vars'!>x)),
+                              GETVAL tmp,
+                              PUSH (VAR sizetmpvar), 
+                              GETVAL (tmp+1),
+                              INFIX (tmp+2) Equal tmp (tmp+1),
+                              PUSH (INT (tmp+2))], labs3)
+          (acode, labs5) <- fcompile' (Assign l (Index (Loc y) (Loc x))) cd' labs4
        -- Compile the body
-	  (bodycode, labs5) <- fcompile' e cd' labs4 
+	  (bodycode, labs6) <- fcompile' e cd' labs5
        -- Point to the next element of ar, jump to start
-	  (inccode, labs6) <- fcompile' 
+	  (inccode, labs7) <- fcompile' 
 	       (Assign (AName x) (Infix Plus (Loc x) (GConst (Num 1)))) 
-	       cd' labs5
-	  return (xcode ++ ycode ++ [LABEL start] ++ testcode ++ [JTRUE end] ++
+	       cd' labs6
+	  return (xcode ++ ycode ++ lencode ++
+                  [LABEL start] ++ testcode ++ [JTRUE end] ++
 		  acode ++ bodycode ++ inccode ++ 
-		  [JUMP start,LABEL end], labs6)
+		  [JUMP start,LABEL end], labs7)
   where	start = MN ("l", lab)
 	end = MN ("l",lab+1)
 	lcx = case uname of
                  Nothing -> MN ("i", lab+2)
                  (Just n) -> n
 	lcy = MN ("i", lab+3)
+        sizetmpvar = ((MN ("stmp",lab+4)),True)
 	vars' = vars ++ [(lcx,True),(lcy,True)]
-	lab' = lab+4
-	sizefn = NS (UN "Builtins") (UN "size")
+	lab' = lab+5
+   -- Array is modified if the local where the array lives in modified, or
+   -- if it's something else we don't know about.
+        arrModified (Loc i) block = elem i (modified block)
+        arrModified (Apply _ _) block = False
+        arrModified (Foreign _ _ _) block = False
+        arrModified (ArrayInit _) block = False
+        arrModified (Annotation a e) block = arrModified e block
+        arrModified _ block = True
 fcompile' (TryCatch tr ca (Loc err) fin) cd@(CD mod vars tmp b trys cache ds vs) lab = 
         do let cd' = (CD mod vars tmp b (trys+1) cache ds vs)
 	   (trcode, lab'') <- fcompile' tr cd' lab'
@@ -421,6 +474,16 @@ fcompile' (TryCatch tr ca (Loc err) fin) cd@(CD mod vars tmp b trys cache ds vs)
    where catchcode = MN ("l",lab)
 	 okay = MN ("l",lab+1)
 	 lab' = lab+2
+fcompile' (NewTryCatch tr cs) cd@(CD mod vars tmp b trys cache ds vs) lab = 
+        do let cd' = (CD mod vars tmp b (trys+1) cache ds vs)
+	   (trcode, lab'') <- fcompile' tr cd' lab'
+	   (catchblocks, lab''') <- catchcompile cs okay cd lab''
+	   let tcode = ((TRY catchcode):trcode) ++ [TRIED, JUMP okay] ++
+		       (LABEL catchcode):catchblocks ++ [LABEL okay]
+	   return (tcode, lab''')
+   where catchcode = MN ("l",lab)
+	 okay = MN ("l",lab+1)
+	 lab' = lab+2
 fcompile' (Throw err) cd lab = 
         do (errcode, lab') <- fcompile' err cd lab
 	   return (errcode ++ [THROW], lab')
@@ -477,7 +540,9 @@ fcompile' (AppendChain xs) cd lab =
           chain [] cd lab = return ([], lab)
 fcompile' (Unary op x) cd lab = icompile False (Unary op x) cd lab
 fcompile' (RealUnary op x) cd lab = rcompile (RealUnary op x) cd lab
-fcompile' (Coerce t1 t2 v) cd lab = cocompile t1 t2 v cd lab
+fcompile' (Coerce t1 t2 v) cd lab = 
+   do (vcode,lab') <- fcompile' v cd lab
+      cocompile t1 t2 vcode cd lab'
 fcompile' (Case v alts) cd lab = 
    do (vcode,lab') <- fcompile' v cd lab
       (acode,lab'') <- altcompile False alts cd lab'
@@ -503,26 +568,64 @@ fcompile' VMPtr cd lab = return ([VMPTR],lab)
 fcompile' (Metavar f l i) cd lab = fail $ f++":"++show l++":Can't compile a metavariable"
 fcompile' (Annotation a e) cd lab =
    do (ecode, lab') <- fcompile' e cd lab
-      annotcompile a ecode cd lab'
+      annotcompile a ecode lab'
 
 fcompile' x _ lab
-    = return (trace ("Warning, unknown parse tree entry " ++ show x) $ ([],lab))
+--    = fail "FOO!"
+    = return (trace ("Warning, unknown parse tree entry, please report:\n" ++ show x) $ ([],lab))
 
 remember [] = []
 remember (x:xs) = (REMEMBER x) : remember xs
 
 --acompile ivs xs = map (\x -> POPARG (x,False)) (map fst xs)
 
+acompile ivs xs = popargs (aget ivs xs)
+
 -- The 'repeat Copy' is in case we don't have the same length information
 -- about argument varness. This is a dodgy hack. Sorry.
-acompile ivs xs = acomp' 0 (zip (ivs++(repeat Copy)) (map fst xs))
+aget ivs xs = acomp' 0 (zip (ivs++(repeat Copy)) (map fst xs))
   where acomp' _ [] = []
         acomp' idx ((v,x):xs) 
-            = (poparg (v, decorate (x,False) idx)) ++ (acomp' (idx+1) xs)
+            = (v, decorate (x,False) idx):(acomp' (idx+1) xs)
+
+--            = (poparg (v, decorate (x,False) idx)) ++ (acomp' (idx+1) xs)
+
+popargs xs = concat $ map poparg xs
 
 poparg (Var, x) = [POPARG x, NOTEVAR x, COPYARG x]
 poparg (Copy, x) = [POPANDCOPYARG x]
 
+--poparg (Var, x) = [DECLAREARG x, NOTEVAR x, COPYARG x]
+--poparg (Copy, x) = [DECLAREARG x, COPYARG x]
+
+
+-- For catchcompile, the exception is already at the top of the stack.
+
+-- No handlers found, rethrow the exception. Rethrow case is inserted
+-- by the parser, but should really be done here!
+catchcompile [] end cd lab = return ([], lab)
+-- If value at top of expression stack is 'ex', execute handler, otherwise
+-- jump to next
+catchcompile ((Catch (Left (ex,args)) handler):xs) end cd lab =
+    do (hcode, lab'') <- fcompile' handler cd lab'
+       (xscode, lab''') <- catchcompile xs end cd lab''
+       let tcode = [JEXNE ex nexthandler] ++ 
+                   bindargs args 0 ++
+                   [RESTORE, TRIED] ++
+                   hcode ++ 
+                   [JUMP end, LABEL nexthandler] ++ xscode
+       return (tcode, lab''')
+   where nexthandler = MN ("l",lab)
+         lab' = lab+1
+         bindargs [] i = []
+         bindargs ((Loc x):xs) i = (EXPROJARG ((locs cd)!>x) i):(bindargs xs (i+1))
+-- Catch all case, don't bother compiling the rest since they won't be used
+catchcompile ((Catch (Right (Loc err)) handler):xs) end cd lab =
+    do (hcode, lab') <- fcompile' handler cd lab
+       let tcode = [(PUSH (VAR ((locs cd)!>err))), SETTOP, RESTORE, TRIED] ++
+                    hcode ++ [JUMP end]
+       return (tcode, lab')
+
 ccompile (Num x) = [PUSH (VAL x)]
 ccompile (Re x) = [PUSH (RVAL x)]
 ccompile (Ch x) = [PUSH (VAL (fromEnum x))]
@@ -553,6 +656,17 @@ ccompile_stack (Str str) lab
 ccompile_stack (Exc str i) lab 
     = return ([PUSH (STR str), PUSH (VAL i), MKEXCEPT], lab)
 
+makeargs :: Monad m => [Expr Name] -> CompData -> Int -> m ([TAC],[Var],Int)
+makeargs [] _ lab = return ([], [], lab)
+makeargs (x:xs) cd lab 
+    = do (pcode, nms, lab') <- makeargs xs cd (lab+1)
+	 (xcode, lab'') <- fcompile' x cd lab'
+	 return (pcode ++ xcode ++ 
+                 [USETMP tmpvar, PUSH (VAR tmpvar), SETTOP], 
+                 nms ++ [tmpvar], 
+                 lab'')
+  where tmpvar = ((MN ("apptmp",lab)), True)
+
 pushargs :: Monad m => [Expr Name] -> CompData -> Int -> m ([TAC],Int)
 pushargs [] _ lab = return ([], lab)
 pushargs (x:xs) cd lab 
@@ -575,6 +689,8 @@ call x cd lab = do (ccode,lab') <- fcompile' x cd lab
 closure (Global n m ar) i left cd lab 
     = return ([CLOSURE (cname n m) i ar],lab)
 closure (Loc l) i left cd lab = return ([CLOSURELOC ((locs cd)!>l) i],lab)
+closure x i left cd lab = do (ccode,lab') <- fcompile' x cd lab
+		             return ((ccode ++ [CLOSURETOP i]),lab')
 
 {-
 closure x cd lab = let (ccode,lab') = fcompile' x cd lab in
@@ -662,39 +778,28 @@ rcompile e cd@(CD mod vars tmp b tr cache ds vs)  lab =
 	 do (xcode, lab') <- fcompile' x (CD mod vars (tmp+1-10000) b tr cache ds vs) lab
 	    return (xcode ++ [GETRVAL tmp], lab', True)
 
-cocompile :: Monad m => Type -> Type -> Expr Name -> CompData -> Int -> 
+cocompile :: Monad m => Type -> Type -> [TAC] -> CompData -> Int -> 
 	                m ([TAC], Int)
-cocompile (Prim StringType) (Prim Number) v cd lab =
-    do (vcode, lab') <- fcompile' v cd lab
-       return (vcode ++ [STR2INT], lab')
-cocompile (Prim Number) (Prim StringType) v cd lab =
-    do (vcode, lab') <- fcompile' v cd lab
-       return (vcode ++ [INT2STR], lab')
-cocompile (Prim RealNum) (Prim StringType) v cd lab =
-    do (vcode, lab') <- fcompile' v cd lab
-       return (vcode ++ [REAL2STR], lab')
-cocompile (Prim StringType) (Prim RealNum) v cd lab =
-    do (vcode, lab') <- fcompile' v cd lab
-       return (vcode ++ [STR2REAL], lab')
-cocompile (Prim StringType) (Prim Character) v cd lab =
-    do (vcode, lab') <- fcompile' v cd lab
-       return (vcode ++ [STR2CHR], lab')
-cocompile (Prim Character) (Prim StringType) v cd lab =
-    do (vcode, lab') <- fcompile' v cd lab
-       return (vcode ++ [CHR2STR], lab')
-cocompile (Prim Boolean) (Prim StringType) v cd lab =
-    do (vcode, lab') <- fcompile' v cd lab
-       return (vcode ++ [BOOL2STR], lab')
-cocompile (Prim Number) (Prim RealNum) v cd lab =
-    do (vcode, lab') <- fcompile' v cd lab
-       return (vcode ++ [INT2REAL], lab')
-cocompile (Prim RealNum) (Prim Number) v cd lab =
-    do (vcode, lab') <- fcompile' v cd lab
-       return (vcode ++ [REAL2INT], lab')
-cocompile (Prim Character) (Prim Number) v cd lab =
-    fcompile' v cd lab -- No conversion needed
-cocompile (Prim Number) (Prim Character) v cd lab =
-    fcompile' v cd lab -- No conversion needed
+cocompile (Prim StringType) (Prim Number) vcode cd lab =
+       return (vcode ++ [STR2INT], lab)
+cocompile (Prim Number) (Prim StringType) vcode cd lab =
+       return (vcode ++ [INT2STR], lab)
+cocompile (Prim RealNum) (Prim StringType) vcode cd lab =
+       return (vcode ++ [REAL2STR], lab)
+cocompile (Prim StringType) (Prim RealNum) vcode cd lab =
+       return (vcode ++ [STR2REAL], lab)
+cocompile (Prim StringType) (Prim Character) vcode cd lab =
+       return (vcode ++ [STR2CHR], lab)
+cocompile (Prim Character) (Prim StringType) vcode cd lab =
+       return (vcode ++ [CHR2STR], lab)
+cocompile (Prim Boolean) (Prim StringType) vcode cd lab =
+       return (vcode ++ [BOOL2STR], lab)
+cocompile (Prim Number) (Prim RealNum) vcode cd lab =
+       return (vcode ++ [INT2REAL], lab)
+cocompile (Prim RealNum) (Prim Number) vcode cd lab =
+       return (vcode ++ [REAL2INT], lab)
+cocompile (Prim Character) (Prim Number) vcode cd lab = return (vcode, lab)
+cocompile (Prim Number) (Prim Character) vcode cd lab = return (vcode, lab)
 cocompile _ _ v _ _ = fail "Internal error, can't coerce"
 
 altcompile :: Monad m => Bool -> -- Put on the heap
@@ -738,7 +843,7 @@ altcompile hp alts cd lab = do (codes, lab') <- mkcase sortalts lab
         getdef ((Default exp):xs) = exp
         getdef (_:xs) = getdef xs
 
-        throwerror = (Throw (GConst (Exc "Missing case" (127))))
+        throwerror = (Throw (Apply (Global missingCase missingCaseMangling 0) []))
 
 -- We have the value we're testing on the top of the stack. Compile
 -- as a jump table.
@@ -762,14 +867,14 @@ constaltcompile hp cs cd lab = ca' cs Number lab []
     mkConstAlts pt acc def cd lab' =
 	return ([CONSTCASE pt acc def], lab')
 
-annotcompile :: Monad m => Annotation -> [TAC] -> CompData -> Int -> 
+annotcompile :: Monad m => Annotation -> [TAC] -> Int -> 
 		m ([TAC], Int)
-annotcompile (Line f l) code cd lab = return $ ((LINENO f l):code, lab)
-annotcompile (FnBody fn f l) code cd lab 
+annotcompile (Line f l) code lab = return $ ((LINENO f l):code, lab)
+annotcompile (FnBody fn f l) code lab 
     = return $ (((PUSHBT fn f l):code++ [POPBT]), lab)
-annotcompile (LamBody f) code cd lab 
+annotcompile (LamBody f) code lab 
     = return $ (((INLAM f):code++[POPBT]), lab)
-annotcompile (DynCheck t) code cd lab = return (code,lab)
+annotcompile (DynCheck t) code lab = return (code,lab)
 
 decls :: [TAC] -> [TAC]
 decls c = add_decls (find_decls c ([],[],[],[],[])) c
@@ -794,6 +899,9 @@ removeothers vs [] = []
 removeothers vs (x:xs) | x `elem` vs = removeothers vs xs
 		       | otherwise = x:(removeothers vs xs)
 
+-- FIXME: Yeeeuuch! There are much neater ways of doing this, probably with
+-- a State monad.
+
 find_decls :: [TAC] -> ([Int],[Int],[Var],[Var],[Var]) -> 
 	               ([Int],[Int],[Var],[Var],[Var])
 find_decls [] (ts,ts',vs,vs',as) = (ts,ts',vs,vs',as)
@@ -835,6 +943,7 @@ find_decls ((CONSTCASE _ ccs defcode):xs) (ts,ts',vs,vs',as)
    where fd_alts [] acc = acc
 	 fd_alts (xs:xss) acc = find_decls xs (fd_alts xss acc)
 find_decls ((PROJ v i):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',as)
+find_decls ((EXPROJARG v i):xs) (ts,ts',vs,vs',as) = find_decls xs (ts,ts',v:vs,vs',as)
 find_decls (_:xs) acc = find_decls xs acc
 
 
@@ -846,6 +955,41 @@ find_decls (_:xs) acc = find_decls xs acc
 -- 	numargs (Fn _ ts t) = length ts
 -- 	numargs _ = 0
 
+mkGlobInit :: Program -> Expr Name
+mkGlobInit [] = Noop
+mkGlobInit ((Glob (n,st,gid,Nothing)):xs) = mkGlobInit xs
+mkGlobInit ((Glob (n,st,gid,Just val)):xs) = 
+    Seq (Assign (AGlob gid) val) (mkGlobInit xs)
+mkGlobInit (_:xs) = mkGlobInit xs
+
+addGlobInit :: Program -> Name -> Program
+addGlobInit prg mod 
+    = let expr = exceptGuard mod $ mkGlobInit prg 
+          ty = Fn [] [] (Prim Void) in
+          (FunBind ("(autogenerated)",0,(NS mod (UN "__init")),
+                    ty,[StartupFn],Defined expr)
+                   "" ty):prg
+
+-- Stick a try...catch block around global initialisation
+exceptGuard mod Noop = Noop
+exceptGuard mod expr = Lambda [] [] $
+  Declare "(autogenerated)" 0 
+     (MN ("exc",0), True) (Prim Exception) $
+       TryCatch expr dumpex (Loc 0) Noop
+      where dumpex = Seq printfail $
+                     Seq 
+                     (Apply (Global backtracefun backtracemangle 1) 
+                        [Loc 0])
+                     bombout
+            bombout = Apply (Global exitfun exitmangle 1) 
+                        [GConst (Num 0)]
+            printfail = Apply (Global putstrlnfun putstrlnmangle 1) 
+                        [GConst (Str "Global initialisation failed")]
+
+conZeroCode t = Code [] [] [MKCONZERO t]
+conCode t i = Code [] [] [MKCON t i]
+exceptCode nm i = Code [] [] [MKNEWEXCEPT nm i]
+
 compileAll :: Monad m => Program -> Name -> m [CompileResult]
 compileAll ds n = ca' ds n
   where
@@ -863,21 +1007,37 @@ compileAll ds n = ca' ds n
     ca' ((FunBind (_,_,n, ty,_,Unbound) _ _):xs) mod = do
         rest <- ca' xs mod
 	return $ (ExternDef (n, ty)):rest
+    ca' ((FunBind (_,_,n, ty,_,ExtInlinable _) _ _):xs) mod = do
+        rest <- ca' xs mod
+	return $ (ExternDef (n, ty)):rest
     ca' ((FunBind (_,_,n,ty,_,DataCon t 0 True) _ _):xs) mod = do
         rest <- ca' xs mod    
-	return $ (ByteCode (n,ty,[MKCONZERO t])):rest
+	return $ (ByteCode (n,ty,conZeroCode t)):rest
     ca' ((FunBind (_,_,n,ty,_,DataCon t i True) _ _):xs) mod = do
         rest <- ca' xs mod    
-	return $ (ByteCode (n,ty,[MKCON t i])):rest
+	return $ (ByteCode (n,ty,conCode t i)):rest
     ca' ((FunBind (_,_,n,ty,_,DataCon t i False) _ _):xs) mod = do
         rest <- ca' xs mod    
 	return $ (ExternDef (n, ty)):rest
+    ca' ((FunBind (_,_,n,ty,_,ExceptionFn nm i True) _ _):xs) mod = do
+        rest <- ca' xs mod
+        return $ (ExcCode nm True):(ByteCode (n,ty,exceptCode nm i)):rest
+    ca' ((FunBind (_,_,n,ty,_,ExceptionFn nm i False) _ _):xs) mod = do
+        rest <- ca' xs mod
+        return $ (ExcCode nm False):(ExternDef (nm ,ty)):rest
 --ca' ((DataDecl n tys cs):xs) = (compileDecls cs) ++ (ca' xs)
-    ca' ((Glob (n,st,gid)):xs) mod = do
+    ca' ((Glob (n,st,gid,_)):xs) mod = do
         rest <- ca' xs mod
 	return $ (GlobCode gid):rest
+--    ca' ((ExceptDecl f l n ty comm):xs) mod = do
+--        rest <- ca' xs mod
+--        let ext = if defhere then "" else "extern "
+--        return $ (RawCode $ ext ++ "void* " ++show n++";\n"):rest
     ca' (x:xs) mod = ca' xs mod
 
+-- 'peephole' optimisation of bytecode; combines common sequences of
+-- instructions into faster instructions.
+
 peephole :: [TAC] -> [TAC]
 peephole [] = []
 -- Add argument declaration.
@@ -962,8 +1122,16 @@ peephole (x:xs) = x:(peephole xs)
 foreigncomp :: Monad m => Type -> Name -> [(Expr Name,Type)] -> CompData -> 
 	       Int -> m ([TAC], Int)
 foreigncomp ty n as cd lab = 
-    do (pcode, lab') <- pushargsHeap (map fst as) cd lab
+    do (pcode, lab') <- fpushargs as cd lab
        return (pcode ++ foreigncall ty n (map snd as), lab')
+  where fpushargs [] _ lab = return ([], lab)
+        fpushargs ((x, ty):xs) cd lab 
+            = do (pcode, lab') <- fpushargs xs cd lab
+	         (xcode, lab'') <- cmpfn ty x cd lab'
+	         return (pcode ++ xcode, lab'')
+        cmpfn (Prim Exception) = fMkHeap
+        cmpfn (Prim _) = fcompile'
+        cmpfn _ = fMkHeap
 
 foreigncall :: Type -> Name -> [Type] -> [TAC]
 {- old way
@@ -1009,6 +1177,17 @@ foreigncall ty (UN n) args = [USETMP ((tmpval n), True) | n <- [0..(length args)
 	 stackconv' _ t = error $ "Can't deal with that type (" ++ show t ++ ") in foreign calls"
 -}
 
+mkStartup :: Program -> CompileResult
+mkStartup p = let startups = callStartups p in
+                RawCode $ "void _my_runatstartup(VMState* vm) {\n " ++ startups ++ "}"
+
+callStartups [] = ""
+callStartups ((FunBind (_,_,n,ty,fopts,_) _ _):xs) 
+  | StartupFn `elem` fopts =
+      "\tCALL(" ++ show n ++ mangling ty ++ ");\n" ++ callStartups xs
+callStartups (_:xs) = callStartups xs
+
+
 type FunMap = [(String,Int)]
 
 mkfnmap :: Program -> CompileResult
@@ -1028,9 +1207,11 @@ mkf' i ((FMName x):xs) = (x, i):(mkf' (i+1) xs)
 mkf' i (x:xs) = mkf' (i+1) xs
 
 mkfinit :: FunMap -> String
-mkfinit [] = ""
-mkfinit ((str,i):[]) = "\tinitFunMap(" ++ show (i+1) ++");\n"
-mkfinit ((str,i):xs) = mkfinit xs
+mkfinit fm = mkfi' fm (hash (show fm))
+  where mkfi' [] h = ""
+        mkfi' ((str,i):[]) h = "\tinitFunMap(" ++ show (i+1) ++", (int)" ++ 
+                               show h ++ ");\n"
+        mkfi' ((str,i):xs) h = mkfi' xs h
 
 mkfcode :: FunMap -> String
 mkfcode [] = ""
@@ -1054,7 +1235,7 @@ mkivec seed = do key <- mkkey (seed /= 0) 32 seed
 
 showtac :: [CompileResult] -> String
 showtac [] = ""
-showtac ((ByteCode (n,ty,b)):xs) = showuser n ++ ": " ++ mangling ty ++ "\n" ++ showbc 1 b ++ 
+showtac ((ByteCode (n,ty,(Code pop _ main))):xs) = showuser n ++ ": " ++ mangling ty ++ "\n" ++ showbc 1 (pop ++ main) ++ 
 				"\n\n" ++ showtac xs
   where showbc i [] = ""
 	showbc i ((CASE bcs):xs) = (take i (repeat '\t')) ++ 
@@ -1064,3 +1245,6 @@ showtac ((ByteCode (n,ty,b)):xs) = showuser n ++ ": " ++ mangling ty ++ "\n" ++
 	showcase i c (bc:bcs) = (take (i-1) (repeat '\t')) ++ show c ++ 
 				":\n" ++ showbc i bc ++ showcase i (c+1) bcs
 showtac (x:xs) = showtac xs
+
+hash [] = 0
+hash (x:xs) = 131*(hash xs)+(fromEnum x)
