@@ -153,22 +153,22 @@ int dowattroff(void* win, KayaArray attribs)
 
 wchar_t dogetch()
 {
-    return chrtowc(getch());
+    return KACHAR(getch());
 }
 
 wchar_t domvgetch(int y, int x)
 {
-    return chrtowc(mvgetch(y,x));
+    return KACHAR(mvgetch(y,x));
 }
 
 wchar_t dowgetch(void* win)
 {
-    return chrtowc(wgetch((WINDOW*)win));
+    return KACHAR(wgetch((WINDOW*)win));
 }
 
 wchar_t dowmvgetch(void* win, int y, int x)
 {
-    return chrtowc(mvwgetch((WINDOW*)win,y,x));
+    return KACHAR(mvwgetch((WINDOW*)win,y,x));
 }
 
 wchar_t* dogetstr()
@@ -256,14 +256,30 @@ DEFKEY(DOWN);
 DEFKEY(UP);
 DEFKEY(LEFT);
 DEFKEY(RIGHT);
-DEFKEY(HOME);
-DEFKEY(END);
 DEFKEY(BACKSPACE);
 DEFKEY(A1);
 DEFKEY(A3);
 DEFKEY(B2);
 DEFKEY(C1);
 DEFKEY(C3);
+DEFKEY(HOME);
+DEFKEY(END);
+DEFKEY(PPAGE);
+DEFKEY(NPAGE);
+
+// function keys
+DEFKEYF(1);
+DEFKEYF(2);
+DEFKEYF(3);
+DEFKEYF(4);
+DEFKEYF(5);
+DEFKEYF(6);
+DEFKEYF(7);
+DEFKEYF(8);
+DEFKEYF(9);
+DEFKEYF(10);
+DEFKEYF(11);
+DEFKEYF(12);
 
 int dobox(void* win,wchar_t x, wchar_t y)
 {
