@@ -10,7 +10,9 @@
 #include <stdfuns.h>
 
 #define DECLKEY(x) wchar_t key_## x();
-#define DEFKEY(x) wchar_t key_## x() { return KCHAR(KEY_## x); }
+#define DEFKEY(x) wchar_t key_## x() { return KACHAR(KEY_## x); }
+#define DECLKEYF(x) wchar_t key_F## x();
+#define DEFKEYF(x) wchar_t key_F## x() { return KACHAR(KEY_F(x)); }
 
 extern "C" {
     bool testerr(int x);
@@ -73,14 +75,29 @@ extern "C" {
     DECLKEY(UP);
     DECLKEY(LEFT);
     DECLKEY(RIGHT);
-    DECLKEY(HOME);
-    DECLKEY(END);
     DECLKEY(BACKSPACE);
     DECLKEY(A1);
     DECLKEY(A3);
     DECLKEY(B2);
     DECLKEY(C1);
     DECLKEY(C3);
+    DECLKEY(HOME);
+    DECLKEY(END);
+    DECLKEY(PPAGE);
+    DECLKEY(NPAGE);
+
+    DECLKEYF(1);
+    DECLKEYF(2);
+    DECLKEYF(3);
+    DECLKEYF(4);
+    DECLKEYF(5);
+    DECLKEYF(6);
+    DECLKEYF(7);
+    DECLKEYF(8);
+    DECLKEYF(9);
+    DECLKEYF(10);
+    DECLKEYF(11);
+    DECLKEYF(12);
 
     int dobox(void* win,wchar_t x, wchar_t y);
 
