@@ -1,6 +1,7 @@
 
 #include "pg_inter.h"
 #include <VMState.h>
+#include <cstdlib>
 #include <KayaAPI.h>
 
 void* pg_connect(wchar_t* rawconninfo)
@@ -70,7 +71,7 @@ void pg_getTypeIDs(VMState* vm, PGCon* pgc)
     }
 }
 
-void* pg_exec(void* vmptr,void* conn,wchar_t* rawquery)
+void* pg_exec(void* vmptr,void* conn,wchar_t* rawquery, KayaValue err)
 {
   char* query = wctostr(rawquery);
     VMState* vm = (VMState*)vmptr;
@@ -80,11 +81,13 @@ void* pg_exec(void* vmptr,void* conn,wchar_t* rawquery)
 
     PGresult *res = PQexec(pgc->con,query);
     if (res==NULL) {
-	vm->kaya_throw(PQerrorMessage(pgc->con),1);
+	KayaSetString(err, KSTRING(PQerrorMessage(pgc->con)));
+	return NULL;
     }
     if ((PQresultStatus(res)!=PGRES_COMMAND_OK)
 	&& (PQresultStatus(res) != PGRES_TUPLES_OK)) {
-	vm->kaya_throw(PQresultErrorMessage(res),1);
+	KayaSetString(err, KSTRING(PQresultErrorMessage(res)));
+	return NULL;
     }
     
     int numrows = PQntuples(res);
@@ -153,7 +156,7 @@ wchar_t* pg_columnname(void* vmptr,void* res, int col)
     VMState* vm = (VMState*)vmptr;
     PGresult *r = ((PGRes*)res)->pgres;
     if (col>=((PGRes*)res)->cols) {
-	vm->kaya_throw("Column number out of range",1);
+	vm->kaya_internalError(3);
     }
     return strtowc(PQfname(r,col));
 }
