@@ -38,7 +38,7 @@ extern "C" {
 /// Returns an error message
     wchar_t* pg_getError(void* conn);
 /// Returns a PGRes
-    void* pg_exec(void* vmptr,void* conn,wchar_t* query);
+    void* pg_exec(void* vmptr,void* conn,wchar_t* query, KayaValue err);
 /// Returns a column name
     wchar_t* pg_columnname(void* vmptr,void* res, int col);
 /// Gets the strings from a PGRes
