@@ -16,25 +16,16 @@
 
 #define ALLOC 1024
 
-Array::Array(int size):m_alloc(ALLOC),m_size(0),m_first(NULL)
+Array::Array(int size):m_size(0),m_first(NULL)
 {
-    if (size!=0) { // i.e. size explicitly given, so expect array to be
-	           // initialised with something.
-      //	m_size=0;
-	m_alloc = size;
-	/* this appears to do absolutely nothing... */
-	// Populate the chunk with empty values
-	//	static Value* empty = new Value(NULL,KVT_NULL);
-	//	for(int i=0;i<m_size;++i)
-	//	    m_first->data[i]=empty;
+    if (size==0) { 
+	size = ALLOC;
     }
-//    m_data = (Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*m_alloc);
-//    m_data = new Value*[m_alloc];
 
     m_first = new chunk();
-    m_first -> data = (Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*m_alloc);
+    m_first -> data = (Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*size);
 
-    m_first -> size = m_alloc;
+    m_first -> size = size;
     m_first -> next = NULL;
 
 }
@@ -89,15 +80,6 @@ void Array::resize(int i)
     }
 // Perhaps delete chunks we no longer need here?
 
-//     if (m_size<=m_alloc) {
-// 	m_alloc=m_size+1;
-// 	Value** newdata = (Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*m_alloc);
-// 	for(int i=0;i<oldsize;i++)
-// 	    newdata[i]=m_data[i];
-// 	for(int i=oldsize;i<m_size;i++)
-// 	    newdata[i]=new Value(NULL,NULL);
-// 	m_data=newdata;
-//     }
 }
 
 Value* Array::shift()
@@ -117,6 +99,16 @@ Value* Array::shift()
     return shiftval;
 }
 
+int Array::reserved_size() {
+  chunk* c = m_first;
+  int sz = 0;
+  while (c!=NULL) {
+      sz += c->size;
+      c = c->next;
+  }
+  return sz;
+}
+
 void Array::push_back(Value *v)
 {
     chunk* c = m_first;
@@ -126,7 +118,7 @@ void Array::push_back(Value *v)
 	if (i<c->size) {
 	    break;
 	} else {
-	    i=i-c->size;
+	    i -= c->size;
 	    last = c;
 	    c = c->next;
 	}
@@ -146,19 +138,8 @@ void Array::push_back(Value *v)
 	i=0;
     }
     c->data[i] = v;
-    m_size++;
+    ++m_size;
 
-/*    if (m_size==m_alloc) {
-	m_alloc*=2;
-	/// CAN'T DO THIS!!!!1!!!
-	Value** newdata = (Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*m_alloc);
-//	Value** newdata = new Value*[m_alloc];
-	for(int i=0;i<m_size;i++)
-	    newdata[i]=m_data[i];
-	m_data=newdata;
-    }
-    m_data[m_size]=v;
-    m_size++;*/
 }
 
 /*
@@ -185,13 +166,13 @@ Value* Array::lookup(int i)
 }
 
 // TODO: These are too slow, since we can peek directly rather than lookup.
-bool Array::eq(Array* x)
+bool Array::eq(Array* x, map<Value*, Value*>& done)
 {
     if (x->size()!=size()) return false;
-    for(int i=0;i<m_size;i++) {
+    for(int i=0;i<m_size;++i) {
 	Value* idx = lookup(i);
 	Value* xidx = x->lookup(i);
-	if (!funtable_eq(NULL,idx,xidx)) return false;
+	if (!funtable_eq_aux(idx,xidx,done)) return false;
     }
     return true;
 }
@@ -200,7 +181,7 @@ int Array::cmp(Array* x)
 {
     if (x->size()<size()) return -1;
     if (x->size()>size()) return 1;
-    for(int i=0;i<m_size;i++) {
+    for(int i=0;i<m_size;++i) {
 	Value* idx = lookup(i);
 	Value* xidx = x->lookup(i);
 	int cmp = funtable_compare(NULL,idx,xidx);
@@ -212,7 +193,7 @@ int Array::cmp(Array* x)
 Array* Array::copy(map<Value*,Value*>& done)
 {
     Array* a = new Array(m_size);
-    for(int i=0;i<m_size;i++) {
+    for(int i=0;i<m_size;++i) {
 	Value* copied;
 	Value* x = lookup(i);
 	// if x is already copied, return the copied value
@@ -236,7 +217,7 @@ int Array::memusage()
     while(c!=NULL) {
 	size+=sizeof(chunk);
 	size+=c->size*sizeof(Value*);
-	for(x=0;x<c->size;x++) {
+	for(x=0;x<c->size;++x) {
 	    if (c->data[x]!=NULL)
 		size+=funtable_memusage(c->data[x]);
 	}
