@@ -39,7 +39,7 @@ public:
     void push_back(Value* v);
     Value* lookup(int i);
     inline int size() { return m_size; }
-    inline int reserved_size() { return m_alloc; }
+    int reserved_size();
     Array* copy(map<Value*,Value*>& done);
     void resize(int i);
     Value* shift();
@@ -50,11 +50,11 @@ public:
     int memusage();
 //    void append(Array* xs);
 
-    bool eq(Array* x);
+    bool eq(Array* x,map<Value*,Value*>& done);
     int cmp(Array* x);
 
 private:
-    int m_alloc;
+    //    int m_alloc;
     int m_size;
     chunk* m_first;
 };
