@@ -39,13 +39,14 @@ public:
     void setArg(int i,Value* v) { m_args[i]=v; }
     void addArgs(VMState* vm, int i);
 
-    bool eq(Closure* c);
+    bool eq(Closure* c,map<Value*,Value*>& done);
     Closure* copy(map<Value*, Value*>& done);
 
 private:
     func m_fn;
     Value** m_args;
     int m_numargs;
+    int m_arity;
 };
 
 #endif
