@@ -29,11 +29,12 @@ Value* zero = new Value((void*)0,KVT_INT);
 Value* one = new Value((void*)1,KVT_INT);
 Value* minusone = new Value((void*)(-1),KVT_INT);
 
-/*
-Value::Value(void* ptr, valtype ft):m_val(ptr),m_funtable(ft)
-{
-//    cout << "Created a value " << this << endl;
-}*/
+wchar_t* emptyString() {
+  wchar_t* buf = (wchar_t*)GC_MALLOC_ATOMIC(sizeof(wchar_t));
+  buf[0] = '\0';
+  return buf;
+}
+
 
 String* Value::getString()
 {
@@ -41,14 +42,19 @@ String* Value::getString()
     if (m_val!=NULL)
 	return (String*)m_val;
     else
-	return new String(L"");
+	return new String(emptyString());
 }
 
 Exception* Value::getExcept()
 {
     /// TODO - throw if it's not an exception.
-    if (m_val!=NULL)
+    if (m_val!=NULL) {
+	if (m_funtable!=KVT_EXCEPTION) {
+	    assert(false);
+	    return NULL;
+	}
 	return (Exception*)m_val;
+    }
     else {
 	assert(false);
 	return NULL;
@@ -62,9 +68,10 @@ void Value::runClosure(VMState* vm)
 //    ((func)m_val)(vm);
 }
 
-void Value::addClosureArgs(VMState *vm, int i)
+Value* Value::addClosureArgs(VMState *vm, int i)
 {
     ((Closure*)m_val)->addArgs(vm,i);
+    return this;
 }
 
 Array* Value::getArray()
@@ -170,48 +177,11 @@ void Value::setArray(Array* a)
     m_funtable = KVT_ARRAY;
 }
 
-void Value::readString()
-{
-    char buf[255];
-    fgets(buf,255,stdin);
-    char *loc = strchr(buf,'\n');
-    *loc = '\0';
-    setString(new String(strtowc(buf)));
-}
-
-void Value::readInt()
-{
-    int v;
-    scanf("%d",&v);
-    setInt(v);
-}
-
-/*
-void Value::setIdxPtr(Value* p,unsigned i)
-{
-//    cout << "Setting " << this << " to " << p->m_val << endl;
-    Value* current=this;
-    for(unsigned x=0;x<i;x++)
-    {
-	--stacksize;
-	int idx = valstack[stacksize]->getInt();
-//	int idx = stack.back()->getInt();
-//	cout << idx << ",";
-//	stack.pop_back();
-	valstack[stacksize]=0;
-	Value* next = current->lookup(idx);
-	current = next;
-    }
-//    cout << endl;
-    current->m_val=p->m_val;
-    current->m_funtable=p->m_funtable;
-}
-*/
 
 void Value::project(VMState* vm, int i)
 {
     if (vm->topItem()->getType()!=KVT_UNION) {
-	vm->kaya_throw(L"Attempt to project from non-Union",1);
+	vm->kaya_rtsError(PROJECT_FROM_NON_UNION);
     }
 
     Union* top=(Union*)(vm->topItem()->m_val);
@@ -220,33 +190,16 @@ void Value::project(VMState* vm, int i)
     m_funtable = arg->m_funtable;
 }
 
-/*
-void Value::makeArray(int i,vector<Value*>& stack)
-{
-    int size=1;
-    for(int x=0;x<i;x++)
-    {
-	int idx = stack.back()->getInt();
-	stack.pop_back();
-	size=size*idx;
-    }
-    m_val=(void*)(new Value*[size]);
-    for(int j=0;j<size;j++) {
-	((Value**)m_val)[j] = new Value();
-    }
-}
-*/
-
 Value* Value::lookup(VMState* vm, int i)
 {
 #ifndef NOCHECK
     if (m_val && getType()!=KVT_ARRAY) {
 //	cout << m_val << "," << getType() << endl;
-	vm->kaya_throw(L"Attempt to lookup in non-Array",1);
+	vm->kaya_rtsError(LOOKUP_FROM_NON_ARRAY);
     }
 #endif
     if (i<0) {
-	vm->kaya_throw(L"Attempt to lookup negative array index",1);
+	vm->kaya_rtsError(NEGATIVE_ARRAY_INDEX);
     }
 
     Array* v = (Array*)m_val;
@@ -261,7 +214,7 @@ Value* Value::lookup(VMState* vm, int i)
     if (extras>=0) {
 	v->expectedSize(i);
 	Value* n = new Value(NULL, KVT_NULL);
-	for(int l=0;l<extras;l++) {
+	for(int l=0;l<extras;++l) {
 	    v->push_back(n);
 	    n = new Value(NULL, KVT_NULL);
 	}
@@ -283,7 +236,7 @@ int Value::length()
     return v->size();
 }
 
-String::String(wchar_t* val)
+String::String(const wchar_t* val):m_offset(0)
 {
     if (val!=NULL) {
 	m_alloc = wcslen(val)*2;
@@ -294,21 +247,19 @@ String::String(wchar_t* val)
 	m_alloc = 128;
 	m_str=(wchar_t*)GC_MALLOC_ATOMIC(m_alloc*sizeof(wchar_t));
 //	strcpy(m_str,"(none)"); // Debugging purposes
-	wcscpy(m_str,L"");
+	wcscpy(m_str,emptyString());
     }
 }
 
-String::String(wchar_t val)
+String::String(wchar_t val):m_alloc(16),m_offset(0)
 {
-    m_alloc = 16;
     m_str=(wchar_t*)GC_MALLOC_ATOMIC(m_alloc*sizeof(wchar_t));
     m_str[0]=val;
     m_str[1]='\0';
 }
 
-String::String(int len)
+String::String(int len):m_alloc(len+1),m_offset(0)
 {
-    m_alloc = len+1; // space for /0
     m_str=(wchar_t*)GC_MALLOC_ATOMIC(m_alloc*sizeof(wchar_t));
     m_str[0]='\0';
 }
@@ -318,12 +269,13 @@ void String::append(String* s)
     // reallocation is safe with strings, because there will be no
     // references into the middle.
     int newlen = length() + s->length();
-    if (newlen>=m_alloc) {
+    if (newlen>=M_ALLOC) {
 	m_alloc = (newlen+8)*2; //(newlen+10)*2;
 //	cout << "Realloc " << m_alloc << " to " << wctostr(m_str) << " + " << wctostr(s->getVal()) << endl;
 	wchar_t* newstr = (wchar_t*)GC_MALLOC_ATOMIC(sizeof(wchar_t)*m_alloc);
-	wcscpy(newstr,m_str);
+	wcscpy(newstr,M_STR);
 	m_str=newstr;
+	m_offset=0;
     }
     wcscat(m_str,s->getVal());
 }
@@ -331,26 +283,28 @@ void String::append(String* s)
 void String::append(wchar_t c)
 {
     int newlen = length()+1;
-    if (newlen>=m_alloc) {
+    if (newlen>=M_ALLOC) {
 	m_alloc = (newlen+8)*2; //(newlen+10)*2;
 //	cout << "Realloc " << m_alloc << " to " << wctostr(m_str) << endl;
 	wchar_t* newstr = (wchar_t*)GC_MALLOC_ATOMIC(sizeof(wchar_t)*m_alloc);
-	wcscpy(newstr,m_str);
+	wcscpy(newstr,M_STR);
 	m_str=newstr;
+	m_offset=0;
     }
     m_str[newlen-1]=c;
     m_str[newlen]='\0';
 }
 
-void String::append(wchar_t* s)
+void String::append(const wchar_t* s)
 {
     int newlen = length() + wcslen(s);
-    if (newlen>=m_alloc) {
+    if (newlen>=M_ALLOC) {
 	m_alloc = (newlen+8)*2; //(newlen+10)*2;
 //	cout << "Realloc " << m_alloc << " to " << wctostr(m_str) << endl;
 	wchar_t* newstr = (wchar_t*)GC_MALLOC_ATOMIC(sizeof(wchar_t)*m_alloc);
-	wcscpy(newstr,m_str);
+	wcscpy(newstr,M_STR);
 	m_str=newstr;
+	m_offset=0;
     }
     wcscat(m_str,s);
 }
@@ -358,25 +312,43 @@ void String::append(wchar_t* s)
 bool String::eq(String* s)
 {
 //    cout << "Comparing " << s->getVal() << " and " << m_str << "." << endl;
-    return (wcscmp(m_str,s->getVal())==0);
+    return (wcscmp(M_STR,s->getVal())==0);
 }
 
 bool String::eq(wchar_t* s)
 {
-    return (wcscmp(m_str,s)==0);
+    return (wcscmp(M_STR,s)==0);
 }
 
 int String::cmp(String* s)
 {
 //    cout << "Comparing " << s->getVal() << " and " << m_str << "." << endl;
-    return (wcscmp(m_str,s->getVal()));
+    return (wcscmp(M_STR,s->getVal()));
+}
+
+void String::offset(int inc)
+{
+    m_offset+=inc;
+    // If it's got too long, reallocate.
+    if (m_offset>OFFSET_LIMIT) {
+	wchar_t* newstr = (wchar_t*)GC_MALLOC_ATOMIC(sizeof(wchar_t)*m_alloc);
+	wcscpy(newstr,M_STR);
+	m_str=newstr;
+	m_offset=0;
+    }
+}
+
+void String::chop(int inc)
+{
+    int len = length();
+    M_STR[len-inc]='\0';
 }
 
 int String::hash()
 {
     int hash = 0;
-    for(unsigned i=0;i<wcslen(m_str);i++) {
-	hash = 131*hash+m_str[i];
+    for(unsigned i=0;i<wcslen(M_STR);++i) {
+	hash = 131*hash+M_STR[i];
     }
     return hash;
 }
@@ -391,7 +363,7 @@ Union::Union(VMState* vm, int t,int ar, bool getargs)
     if (ar > 0) {
       args = (Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*ar);
       if (getargs) {
-	for(int i=0;i<ar;i++) {
+	for(int i=0;i<ar;++i) {
 	  //	    --stacksize;
 	  /// Create a *new* reference, or things break in confusing ways.
 	  args[i]=vm->doPop()->clone();
@@ -402,12 +374,12 @@ Union::Union(VMState* vm, int t,int ar, bool getargs)
     }
 }
 
-bool Union::eq(Union* x)
+bool Union::eq(Union* x, map<Value*, Value*>& done)
 {
     if (THIS_TAG!=U_TAG(x)) return false;
-    for(int i=0;i<THIS_ARITY;i++)
+    for(int i=0;i<THIS_ARITY;++i)
     {
-	if (!funtable_eq(NULL,args[i],x->args[i])) { return false; }
+	if (!funtable_eq_aux(args[i],x->args[i],done)) { return false; }
     }
     return true;
 }
@@ -417,7 +389,7 @@ int Union::cmp(Union* x)
     if (THIS_TAG<U_TAG(x)) return -1;
     if (THIS_TAG>U_TAG(x)) return 1;
 
-    for(int i=0;i<THIS_ARITY;i++)
+    for(int i=0;i<THIS_ARITY;++i)
     {
 	int cmp = funtable_compare(NULL,args[i],x->args[i]);
 	if (cmp!=0) return cmp;
@@ -428,7 +400,7 @@ int Union::cmp(Union* x)
 Union* Union::copy(map<Value*,Value*>& done)
 {
     Union* u = new Union(NULL,THIS_TAG,THIS_ARITY,false);
-    for(int i=0;i<THIS_ARITY;i++) {
+    for(int i=0;i<THIS_ARITY;++i) {
 	Value* copied;
 	Value* x = args[i];
 	// if x is already copied, return the copied value
@@ -469,9 +441,35 @@ Exception::Exception(VMState* vm)
     }
 }
 
+Exception::Exception(VMState* vm, char* t, int ar):
+    tag(t), arity(ar)
+{
+    err = new String(strtowc(t));
+    code = 0;
+    if (ar > 0) {
+	args = (Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*ar);
+	for(int i=0;i<ar;++i) {
+	    //	    --stacksize;
+	    /// Create a *new* reference, or things break in confusing ways.
+	    args[i]=vm->doPop()->clone();
+	}
+    } else {
+	args = NULL;
+    }
+
+    throwfile = vm->m_sourcefile;
+    throwline = vm->m_lineno;
+
+    CallStackEntry** c = vm->m_btptr-1;
+    for(;(c>vm->m_backtrace);--c) {
+	CallStackEntry nc(**c);
+	backtrace.push_back(nc);
+    }
+}
+
 void Exception::show()
 {
-    cout << wctostr(err->getVal()) << " (error code " << code << ")" << endl;
+    cout << wctostr(err->getVal()) << endl;
 }
 
 bool Exception::eq(Exception* x)
@@ -492,7 +490,7 @@ int Exception::cmp(Exception* x)
 void Exception::dumpBacktrace()
 {
     vector<CallStackEntry>::iterator it = backtrace.begin();
-    cout << wctostr(err->getVal()) << " (error code " << code << ")";
+    cout << wctostr(err->getVal());
     if (throwline!=0) {
 	cout << " thrown at " << wctostr(throwfile) << ":" << throwline << endl;
     }
