@@ -29,6 +29,7 @@ using namespace std;
 
 //#include <gc/gc_allocator.h>
 #include <vector>
+#include <cstdlib>
 #include "sizes.h"
 #include "ValueFuns.h"
 
@@ -54,6 +55,11 @@ public:
 // cached values
 extern Value *zero, *one, *minusone;
 
+extern "C" {
+  wchar_t* emptyString();
+}
+
+
 #define U_TAG(x) ((x->tag_arity) >> 16)
 #define U_ARITY(x) ((x->tag_arity) & 65535)
 #define THIS_TAG (tag_arity >> 16)
@@ -66,7 +72,7 @@ public:
     Union(VMState* vm, int tag,int arity, bool getargs = true);
     ~Union() { if (args != NULL) { GC_FREE(args); } }
 
-    bool eq(Union* x);
+    bool eq(Union* x, map<Value*,Value*>& done);
     int cmp(Union* x);
     int memusage();
     Union* copy(map<Value*,Value*>& done);
@@ -75,6 +81,39 @@ public:
     Value** args;
 };
 
+struct CallStackEntry : public gc {
+    wchar_t* fn_name;
+    wchar_t* file;
+    int line;
+    wchar_t* call_file;
+    int call_line;
+};
+
+class Exception : public gc
+{
+public:
+    Exception(VMState* vm);
+    Exception(VMState* vm, char* tag, int arity);
+    void show();
+    bool eq(Exception* x);
+    int cmp(Exception* x);
+
+    void dumpBacktrace();
+    
+    String* err;
+    int code;
+
+    char* tag;
+    int arity;
+    Value** args;
+
+    wchar_t* throwfile;
+    int throwline;
+
+    // Need a copy of entire backtrace. Ick.
+    vector<CallStackEntry> backtrace;
+};
+
 class Value : public gc
 {
 public:
@@ -120,7 +159,7 @@ public:
     void runClosure(VMState *vm);
 
     /// Add more arguments from the stack to the closure stored here.
-    void addClosureArgs(VMState *vm, int i);
+    Value* addClosureArgs(VMState *vm, int i);
 
     /// Coercions
     void int2str();
@@ -169,9 +208,6 @@ public:
 
     void setArray(Array* a);
 
-    void readString();
-    void readInt();
-
     /// Set a raw pointer.
     void setPtr(Value* p) {
 //    cout << "Making " << this << " to " << p->m_val << endl;
@@ -200,9 +236,6 @@ public:
 	m_val=(void*)((kint)m_val/(kint)(v->m_val));
     }
 
-    /// Set an array element.
-//    void setIdxPtr(Value* p,unsigned i);
-
     /// Set a value from the projection of the top stack item
     void project(VMState* vm, int i);
     void fastproject(Value* topitem, int i) {
@@ -211,13 +244,14 @@ public:
 	m_val=arg->m_val;
 	m_funtable = arg->m_funtable;
     }
+    void exceptionproject(Value* topitem, int i) {
+	Exception* top=(Exception*)(topitem->m_val);
+	Value* arg=top->args[i];
+	m_val=arg->m_val;
+	m_funtable = arg->m_funtable;
+    }
 
     void fastproject(VMState* vm, int i);
-    /// Project an argument out directly.
-//    void projarg(int i, int t);
-
-    /// Initialise an array with dimensions taken from the stack.
-//    void makeArray(int i,vector<Value*>& stack);
 
     /// Lookup a value in an array --- resize/create if necessary.
     Value* lookup(VMState* vm, int i);
@@ -232,60 +266,46 @@ private:
     valtype m_funtable;
 };
 
+// Get the actual string, considering offset
+#define M_STR (m_str+m_offset)
+// Get the actual length available, considering offset
+#define M_ALLOC (m_alloc-m_offset)
+// Prepared to waste 8k before reallocating a string (this is quite a lot...)
+#define OFFSET_LIMIT 8192
+
 class String : public gc
 {
 public:
-    String(wchar_t* val);
+    String(const wchar_t* val);
     String(wchar_t val);
     String(int len);
 
-    wchar_t* getVal() { return m_str; }
+    wchar_t* getVal() { return M_STR; }
 
-    int length() { return wcslen(m_str); }
-    wchar_t getIndex(int i) { return m_str[i]; }
-    void setIndex(int i,wchar_t c) { m_str[i]=c; }
+    int length() { return wcslen(M_STR); }
+    wchar_t getIndex(int i) { return M_STR[i]; }
+    void setIndex(int i,wchar_t c) { M_STR[i]=c; }
     void append(String* s);
     void append(wchar_t c);
-    void append(wchar_t* s);
+    void append(const wchar_t* s);
     bool eq(String* s);
     bool eq(wchar_t* s);
     int cmp(String* s);
+    // Increase the offset. If it goes over OFFSET_LIMIT, reallocate
+    void offset(int x);
+    // Chop last x characters off.
+    void chop(int x);
     int hash();
 
   // debug/marshalling purposes
-    int space() { return m_alloc; }
+    int space() { return M_ALLOC; }
 
 private:
     wchar_t* m_str;
     int m_alloc;
-};
-
-struct CallStackEntry : public gc {
-    wchar_t* fn_name;
-    wchar_t* file;
-    int line;
-    wchar_t* call_file;
-    int call_line;
-};
-
-class Exception : public gc
-{
-public:
-    Exception(VMState* vm);
-    void show();
-    bool eq(Exception* x);
-    int cmp(Exception* x);
-
-    void dumpBacktrace();
-    
-    String* err;
-    int code;
-
-    wchar_t* throwfile;
-    int throwline;
-
-    // Need a copy of entire backtrace. Ick.
-    vector<CallStackEntry> backtrace;
+    int m_offset; // How much of the start of the string to ignore
+                  // (Optimisation if we're just chopping off the start of
+                  //  the string)
 };
 
 #endif
