@@ -11,7 +11,7 @@
 #include "Array.h"
 #include "VM.h"
 #include "VMState.h"
-
+#include <cstdlib>
 #include <stdarg.h>
 
 KValue newKayaValue()
@@ -26,6 +26,13 @@ KValue KayaInt(kint x)
     return v;
 }
 
+KValue KayaPtr(void* x)
+{
+    KValue v = newKayaValue();
+    KayaSetInt(v,(kint)x);
+    return v;
+}
+
 KValue KayaChar(wchar_t c)
 {
     KValue v = newKayaValue();
@@ -146,11 +153,11 @@ KValue KayaCall(KValue fn, int args, ...)
     va_start(argp,args);
     int i;
     // Gah! Need to push backwards!
-    for(i=0;i<args;i++) {
+    for(i=0;i<args;++i) {
 	arglist[args-i-1] = va_arg(argp,Value*);
     }
     va_end(argp);
-    for(i=0;i<args;i++) {
+    for(i=0;i<args;++i) {
 	PUSH(arglist[i]);
     }
     delete arglist;
@@ -210,7 +217,7 @@ int orArray(KArray a, int(*func)(KValue))
 {
     int val = 0;
     int size = KayaArraySize(a);
-    for(int i=0;i<size;i++) {
+    for(int i=0;i<size;++i) {
 	KValue v = KayaArrayLookup(a,i);
 	val = val | func(v);
     }
