@@ -39,6 +39,8 @@ typedef double KFloat;
 #define KSTRING(x) strtowc(x)
 // char to KChar
 #define KCHAR(x) chrtowc(x)
+// curses alt-chars to KChar
+#define KACHAR(x) (wchar_t)x
 // KString to char*
 #define CSTRING(x) wctostr(x)
 // KChar to char
@@ -55,11 +57,13 @@ typedef double KFloat;
 extern "C" {
     /// Make a new, uninitialised value.
     KValue newKayaValue();
-    /// Make a new, empty array.
+    /// Make a new, empty array, with room for <size> elements.
+    /// Give size 0 to let the rts choose the size.
     KArray newKayaArray(int size);
 
     // Make new, initialised values.
     KValue KayaInt(KInt x);
+    KValue KayaPtr(void* x);
     KValue KayaChar(KChar c);
     KValue KayaFloat(KFloat f);
     KValue KayaString(KString str);
