@@ -23,16 +23,6 @@ using namespace std;
 #include <iostream>
 #include <stack>
 
-/*struct stackdata: public gc {
-    Value** data;
-    int size;
-    };*/
-
-/*extern Value** valstack;
-extern stack<jmp_buf*> except_stack;
-extern stack<int> stack_stack;
-extern int stacksize;
-*/
 
 class VMState;
 
@@ -42,7 +32,7 @@ class VMState;
 
 #define DECLARE(x) Value *x = new Value(NULL,KVT_INT);
 // Args get initialised immediately anyway
-#define DECLAREARG(x) Value *x
+#define DECLAREARG(x) Value stack_##x(NULL,KVT_INT);
 #define DECLAREQUICK(x) Value stack_##x(NULL,KVT_INT); Value* x = &stack_##x;;
 #define HEAPVAL(x) if (x==&stack_##x) { x = new Value(stack_##x.getRaw(),stack_##x.getFunTable()); } 
 // cout << "Made heap val of " << &stack_##x << " -> " << x << endl; cout << x->getRaw() << endl; }
@@ -60,6 +50,7 @@ class VMState;
 #define EMPTYSTR (new Value((void*)(new String((wchar_t*)NULL)),KVT_STRING))
 #define MKUNION(x) (new Value((void*)x,KVT_UNION))
 #define MKCON(t,a) vm->push(new Value((void*)(new Union(vm,t,a)),KVT_UNION))
+#define MKNEWEXCEPT(e,a) vm->push(new Value((void*)(new Exception(vm,e,a)),KVT_EXCEPTION))
 #define MKCONZERO(t) static Union* con = new Union(vm,t,0); vm->push(new Value((void*)con,KVT_UNION))
 #define MKEXCEPT vm->push(new Value((void*)(new Exception(vm)),KVT_EXCEPTION))
 //#define SET(x,i,y) x->setIdxPtr(y,i);
@@ -81,6 +72,7 @@ class VMState;
 
 #define CLOSURE(x,i,tot) vm->push(new Value((void*)(new Closure(vm,x,i,tot)),KVT_FUNC))
 #define CLOSURELOC(x,i) x->addClosureArgs(vm, i); vm->push(x);
+#define CLOSURETOP(i) vm->push(vm->doPop()->addClosureArgs(vm,i));
 
 //#define GETFUN(x) (x->getFunc())
 #define GETVAL(t) (t=vm->doPop()->getInt())
@@ -123,8 +115,6 @@ class VMState;
 #define PRINTINT cout << vm->doPop()->getInt();
 #define PRINTSTR vm->writestdout();
 #define PRINTEXC vm->doPop()->getExcept()->show();
-#define INPUTINT vm->readInt();
-#define INPUTSTR vm->readStr();
 #define NEWLINE cout << endl
 #define LABEL(x) x:
 #define JUMP(x) goto x
@@ -136,6 +126,7 @@ class VMState;
 #define JTRUE(x) if (vm->doPop()->getInt()!=0) goto x
 #define JTFALSE(t,x) if (t==0) goto x
 #define JTTRUE(t,x) if (t!=0) goto x
+#define JEXNE(ex,l) if (vm->topItem()->getExcept()->tag != ex) goto l
 
 #define PUSH(x) vm->push(x)
 #define PUSH2(x,y) vm->push2(x,y)
@@ -171,6 +162,7 @@ class VMState;
 #define APPENDTOPINT vm->appendTopInt();
 #define PROJ(x,i) x->fastproject(vm->topItem(),i);
 #define PROJARG(a,t) vm->projarg(a,t);
+#define EXPROJARG(x,i) x->exceptionproject(vm->topItem(),i);
 /* CIM 12/7/05: changed ERROR to KERROR for MinGW compatibility */
 #define KERROR cerr << vm->doPop()->getString()->getVal(); exit(-1);
 
@@ -229,12 +221,14 @@ class VMState;
    static Value* cache_val; \
    if (cache_val = cache.checkCache(x)) { vm->push(cache_val); return; }
 */
+#define RTCHECKS 0
 #else
 //#define LINENO(f,l) vm->memstat(); vm->lineno(f,l)
 #define LINENO(f,l) vm->lineno(f,l)
 #define PUSHBT(fn,f,l) vm->pushBT(fn,f,l)
 #define INLAM(f) vm->pushBT(f,f,-1)
 #define POPBT vm->popBT()
+#define RTCHECKS 1
 /*
 #define CHECKCACHE(x) static FnCache cache(vm); \
    static Value* cache_val; \
@@ -245,44 +239,6 @@ class VMState;
 VMState* initstack();
 void finish(VMState* vm);
 
-//#define POPINDEX(x) x->setindex(doPop()->getInt(),doPop())
-
-// Value* doPop();
-// void push(Value* val);
-// void mkArray(int size);
-// /// Replace the top stack item with its ith argument.
-// void projarg(int i, int t);
-// void goToIndex();
-// /// Replace the top stack item with the contents of the second stack item.
-// void setTop();
-
-// void pushglobal(char* modid, int i);
-// void createglobal(char* modid,int i);
-
-// int tag();
-
-// void readInt();
-// void readStr();
-// void doAppend();
-// void doEqExcept(bool inv);
-// void doEqString(bool inv);
-
-// void str2int();
-// void int2str();
-// void real2str();
-// void str2real();
-// void str2chr();
-// void chr2str();
-// void bool2str();
-
-// void newjmp_buf();
-
-// void addToFunMap(int id, func fn);
-// func getFn(int id);
-// int getFnID(func fn);
-
-//void kaya_throw(char* msg, int code);
-// void getindex();
 
 Value* mkstr(wchar_t* str);
 // shouldn't be needed?
