@@ -18,12 +18,19 @@
 /// function map is global to all threads.
 func* m_funmap[1];
 int m_funmapsize;
+int m_funmaphash;
 
-void initFunMap(int sz)
+// Defined by compiling Builtins.k, so obviously any binary of a kaya
+// program needs to be linked to at least Builtins.o!
+#define INTERNAL_ERROR _ns__D_Builtins__D_InternalError
+extern char* INTERNAL_ERROR;
+
+void initFunMap(int sz, int fmhash)
 {
     func* funcs = (func*)malloc(sizeof(func)*sz);
     m_funmap[0] = funcs;
     m_funmapsize = sz;
+    m_funmaphash = fmhash;
 }
 
 void addToFunMap(int id, func fn)
@@ -40,12 +47,17 @@ func getFn(int id)
 int getFnID(func fn)
 {
     int i;
-    for (i=1;i<m_funmapsize;i++) {
+    for (i=1;i<m_funmapsize;++i) {
 	if (m_funmap[0][i] == fn) { return i; }
     }
     return -1;
 }
 
+int getFnHash()
+{
+    return m_funmaphash;
+}
+
 VMState::VMState(bool panic):m_return(NULL,KVT_NULL) {
     m_stackalloc=100;
     m_globalloc=100;
@@ -120,26 +132,12 @@ int VMState::tag()
 {
 #ifndef NOCHECK
     if ((*(m_stackptr-1))->getType()!=KVT_UNION) {
-	kaya_throw("Attempt to get tag from non-Union",1);
+	kaya_rtsError(GETTING_TAG_FROM_NON_UNION);
     }
 #endif
     return (*(m_stackptr-1))->getTag();
 }
 
-void VMState::readInt()
-{
-    Value* v=new Value(NULL,KVT_INT);
-    v->readInt();
-    push(v);
-}
-
-void VMState::readStr()
-{
-    Value* v=new Value(NULL,KVT_STRING);
-    v->readString();
-    push(v);
-}
-
 void VMState::doAppend()
 {
     Value* str2=doPop();
@@ -153,12 +151,6 @@ void VMState::doAppendChar(wchar_t x)
 {
     // append x onto top stack item
     (*(m_stackptr-1))->getString()->append(x);
-
-//    Value* str2=doPop();
-//    Value* str1=doPop();
-//    String* str=new String(str1->getString()->getVal());
-//    str->append(x);
-//    push(new Value((void*)str,stringtable));
 }
 
 void VMState::doAppendStr(wchar_t* x)
@@ -166,10 +158,6 @@ void VMState::doAppendStr(wchar_t* x)
     // append x onto top stack item
     (*(m_stackptr-1))->getString()->append(x);
 
-//    Value* str1=doPop();
-//    String* str=new String(str1->getString()->getVal());
-//    str->append(x);
-//    push(new Value((void*)str,stringtable));
 }
 
 void VMState::doAppendStr(char* x)
@@ -306,7 +294,7 @@ void VMState::mkArray(int i)
     // leave a bit of room for expansion (costs a little memory)
     while(i>0) {
 	Value* v= doPop();
-	array->push_back(v);
+	array->push_back(v->clone());
 	i--;
     }
 
@@ -324,7 +312,7 @@ void VMState::pushglobal(Value**& globtable, int i)
 {
     if (globtable == NULL) {
 	globtable = (Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*m_globalloc);
-	for(int i=0;i<m_globalloc;i++) { //TMP HACK
+	for(int i=0;i<m_globalloc;++i) { //TMP HACK
 	    globtable[i]=new Value(0,KVT_INT);
 	}
     }
@@ -336,26 +324,6 @@ void VMState::createglobal(wchar_t* modid, int i)
 {
     cerr << "Do not run this function again" << endl;
     exit(1);
-//     Value** mglobs;
-//     if (m_globs.find(modid)==m_globs.end()) {
-// 	mglobs = (Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*m_globalloc);
-// 	for(int i=0;i<m_globalloc;i++) { //TMP HACK
-// 	    mglobs[i]=new Value(0,NULL);
-// 	}
-// 	m_globs[modid]=mglobs;
-//     }
-//     else {
-// 	mglobs = m_globs[modid];
-//     }
-//     if (i>=m_globalloc)
-//     {
-// 	Value** newglobs = (Value**)GC_MALLOC_UNCOLLECTABLE(sizeof(Value*)*m_globalloc*2);
-// 	for(int i=0;i<m_globalloc;i++)
-// 	    newglobs[i]=mglobs[i];
-// 	mglobs=newglobs;
-// 	m_globalloc*=2;
-// 	m_globs[modid]=mglobs;
-//     }
 }
 
 void VMState::newjmp_buf()
@@ -383,8 +351,23 @@ void VMState::kaya_throw(wchar_t* msg,int code)
 
 void VMState::kaya_throw(char* msg,int code)
 {
-  // CIM: call the wchar_t version to make it easier to put strings in code
-  kaya_throw(strtowc(msg),code);
+    // CIM: call the wchar_t version to make it easier to put strings in code
+    kaya_throw(strtowc(msg),code);
+}
+
+void VMState::kaya_internalError(int code)
+{
+    push(mkint((void*)code));
+    push(new Value((void*)(new Exception(this, INTERNAL_ERROR, 1)), 
+		   KVT_EXCEPTION));
+    throw_ex();
+}
+
+void VMState::kaya_rtsError(char* exceptionID)
+{
+    push(new Value((void*)(new Exception(this, exceptionID, 0)), 
+		   KVT_EXCEPTION));
+    throw_ex();
 }
 
 void VMState::throw_ex()
@@ -428,15 +411,16 @@ Value* VMState::mkint(void* i)
     return new Value(i,KVT_INT);
 }
 
-void VMState::lineno(char* src, int ln) {
+void VMState::lineno(const char* src, int ln) {
   VMState::lineno(strtowc(src),ln);
 }
 
-void VMState::lineno(wchar_t* src, int ln)
+void VMState::lineno(const wchar_t* src, int ln)
 {
 #ifndef NOCHECK
-    m_sourcefile = src; //(char*)KayaAlloc(strlen(src)+1);
-//    strcpy(m_sourcefile,src);
+  // copy is necessary again
+    m_sourcefile = (wchar_t*)GC_MALLOC_ATOMIC(sizeof(wchar_t)*(wcslen(src)+1));
+    wcscpy(m_sourcefile,src);
     m_lineno = ln;
 #endif
 }
@@ -456,9 +440,9 @@ void VMState::pushBT(wchar_t* fn, wchar_t* mod, int ln)
     c->line = ln;
     c->call_line = m_lineno;
 
-    m_btptr++;
+    ++m_btptr;
     if ((m_btptr-m_backtrace)>=m_stackalloc*10) {
-	kaya_throw("Call stack overflow",1);
+	kaya_rtsError(CALL_STACK_OVERFLOW);
     }
 #endif
 }
