@@ -26,13 +26,48 @@ using namespace std;
 
 #include "Heap.h"
 
+// Defined by compiling Builtins.k, so obviously any binary of a kaya
+// program needs to be linked to at least Builtins.o!
+#define PROJECT_FROM_NON_UNION _ns__D_Builtins__D_Project_UN_From_UN_Non_UN_Union
+#define LOOKUP_FROM_NON_ARRAY _ns__D_Builtins__D_Lookup_UN_From_UN_Non_UN_Array
+#define NEGATIVE_ARRAY_INDEX _ns__D_Builtins__D_Negative_UN_Array_UN_Index
+#define WRONG_CONSTRUCTOR _ns__D_Builtins__D_Wrong_UN_Constructor
+#define GETTING_TAG_FROM_NON_UNION _ns__D_Builtins__D_Getting_UN_Tag_UN_From_UN_Non_UN_Union
+#define CALL_STACK_OVERFLOW _ns__D_Builtins__D_Call_UN_Stack_UN_Overflow
+#define INVALID_MARSHALLING_ID _ns__D_Builtins__D_Invalid_UN_Marshalling_UN_ID
+#define INVALID_FUNCTION_TABLE_HASH _ns__D_Builtins__D_Invalid_UN_Function_UN_Table_UN_Hash
+#define CANT_MARSHAL_EXCEPTIONS _ns__D_Builtins__D_Cant_UN_Marshal_UN_Exceptions
+#define CANT_REFLECT_EXCEPTIONS _ns__D_Builtins__D_Cant_UN_Reflect_UN_Exceptions
+#define INVALID_VALUE _ns__D_Builtins__D_Invalid_UN_Value
+#define NOT_IMPLEMENTED _ns__D_Builtins__D_Not_UN_Implemented
+#define INVALID_CIRCULAR_STRUCTURE _ns__D_Builtins__D_Invalid_UN_Circular_UN_Structure
+#define UNKNOWN_FUNCTION_ID _ns__D_Builtins__D_Unknown_UN_Function_UN_ID
+#define DIVIDE_BY_ZERO _ns__D_Builtins__D_Divide_UN_By_UN_Zero
+
+extern char* PROJECT_FROM_NON_UNION;
+extern char* LOOKUP_FROM_NON_ARRAY;
+extern char* NEGATIVE_ARRAY_INDEX;
+extern char* WRONG_CONSTRUCTOR;
+extern char* GETTING_TAG_FROM_NON_UNION;
+extern char* CALL_STACK_OVERFLOW;
+extern char* INVALID_MARSHALLING_ID;
+extern char* INVALID_FUNCTION_TABLE_HASH;
+extern char* INVALID_VALUE;
+extern char* NOT_IMPLEMENTED;
+extern char* INVALID_CIRCULAR_STRUCTURE;
+extern char* CANT_MARSHAL_EXCEPTIONS;
+extern char* CANT_REFLECT_EXCEPTIONS;
+extern char* UNKNOWN_FUNCTION_ID;
+extern char* DIVIDE_BY_ZERO;
+
 /// Function map is global to all threads.
 //extern map<int,func> m_funmap;
 
-void initFunMap(int sz);
+void initFunMap(int sz, int fmhash);
 void addToFunMap(int id, func fn);
 func getFn(int id);
 int getFnID(func fn);
+int getFnHash();
 
 class Exception;
 
@@ -56,7 +91,7 @@ public:
 	// FIXME: Grow stack if necessary? Might be quite an overhead... maybe 
 	// make it an option.
 	*m_stackptr = val;
-	m_stackptr++;
+	++m_stackptr;
     }
 
     // Next ones are for efficiency
@@ -87,19 +122,19 @@ public:
       Value*& topval = (*(m_stackptr-1));
 #ifndef NOCHECK
       if (topval==NULL) {
-	kaya_throw("Projecting from uninitialised value",1);
+	  kaya_rtsError(INVALID_VALUE);
       }
       if (topval->getType()!=KVT_UNION) {
-	kaya_throw("Attempt to project from non-Union",1);
+	  kaya_rtsError(PROJECT_FROM_NON_UNION);
       }
 #endif
       Union* top = (Union*)topval->getRaw();
 #ifndef NOCHECK
       if (U_TAG(top)!=t) {
-	kaya_throw("Wrong constructor used for projection",1);
+	  kaya_rtsError(WRONG_CONSTRUCTOR);
       }
       if (U_ARITY(top) <= i) {
-	kaya_throw("Field out of range in projection; maybe a type error in unmarshal?",1);
+	  kaya_rtsError(INVALID_VALUE);
       }
 #endif
       topval=top->args[i];
@@ -180,8 +215,6 @@ public:
 
     int tag();
 
-    void readInt();
-    void readStr();
     void doAppend();
     void doAppendChar(wchar_t x);
     void doAppendStr(wchar_t* x);
@@ -207,6 +240,8 @@ public:
 
     void kaya_throw(wchar_t* msg, int code);
     void kaya_throw(char* msg, int code);
+    void kaya_internalError(int code);
+    void kaya_rtsError(char* exceptionID);
     void throw_ex();
     jmp_buf* top_ex();
     void tried();
@@ -225,8 +260,8 @@ public:
     Value* mkstr(wchar_t* str);
     Value* mkint(void* i);
 
-    void lineno(char* src, int ln); 
-    void lineno(wchar_t* src, int ln); 
+    void lineno(const char* src, int ln); 
+    void lineno(const wchar_t* src, int ln); 
     void pushBT(char* src, char* mod, int ln);
     void pushBT(wchar_t* src, wchar_t* mod, int ln);
     void popBT();
@@ -248,7 +283,7 @@ public:
 
     // Throw divide by zero exception
     void divideByZero() {
-	kaya_throw("Divide by zero",0);
+	kaya_rtsError(DIVIDE_BY_ZERO);
     }
 
 private:
