@@ -25,7 +25,12 @@ FunTable* realtable = new RealTable();
 
 /// Functions on int
 
-kint inttable_eq(Value* x, Value* y)
+kint inttable_eq(Value* x, Value* y, map<Value*, Value*>& done)
+{
+    return (x->getInt()==y->getInt());
+}
+
+kint inttable_fasteq(Value* x, Value* y)
 {
     return (x->getInt()==y->getInt());
 }
@@ -52,11 +57,19 @@ Value* inttable_marshal(VMState* vm,vector<Value*>& done,Value* x, int id)
 {
     Value* v = new Value(NULL,KVT_STRING);
     wchar_t buf[50];
-    SWPRINTF(buf,50,L"[%d]I[%d]",id,x->getInt());
+    SWPRINTF(buf,50,L"I[%d]",x->getInt());
     v->setString(new String(buf));
     return v;
 }
 
+Value* inttable_reflect(VMState* vm,map<KayaValue, KayaValue>& done,Value* x)
+{
+    Value* v = KayaUnion(0,1);
+    Value* i = KayaInt(x->getInt());
+    KayaUnionSetArg(v,0,i);
+    return v;
+}
+
 Value* inttable_hash(Value* x)
 {
     return new Value((void*)(x->getInt()),KVT_INT);
@@ -69,7 +82,12 @@ int inttable_memusage(Value* x)
 
 /// Functions on String
 
-kint stringtable_eq(Value* x, Value* y)
+kint stringtable_eq(Value* x, Value* y, map<Value*, Value*>& done)
+{
+    return (x->getString()->eq(y->getString()));
+}
+
+kint stringtable_fasteq(Value* x, Value* y)
 {
     return (x->getString()->eq(y->getString()));
 }
@@ -102,14 +120,22 @@ Value* stringtable_marshal(VMState* vm,vector<Value*>& done,Value* x, int id)
     wchar_t buf[sz];
     //    cout <<  << endl;
 #ifdef WIN32
-    SWPRINTF(buf,sz,L"[%d]S[[%d][%d]%s]",id,len,s->space(),s->getVal());
+    SWPRINTF(buf,sz,L"S[[%d][%d]%s]",len,s->space(),s->getVal());
 #else
-    SWPRINTF(buf,sz,L"[%d]S[[%d][%d]%S]",id,len,s->space(),s->getVal());
+    SWPRINTF(buf,sz,L"S[[%d][%d]%S]",len,s->space(),s->getVal());
 #endif
     v->setString(new String(buf));
     return v;
 }
 
+Value* stringtable_reflect(VMState* vm,map<KayaValue, KayaValue>& done,Value* x)
+{
+    Value* v = KayaUnion(2,1);
+    Value* s = KayaString(x->getString()->getVal());
+    KayaUnionSetArg(v,0,s);
+    return v;
+}
+
 Value* stringtable_hash(Value* x)
 {
     return new Value((void*)x->getString()->hash(),KVT_INT);
@@ -122,9 +148,14 @@ int stringtable_memusage(Value* x)
 
 /// Functions on closures
 
-kint fntable_eq(Value* x, Value* y)
+kint fntable_eq(Value* x, Value* y, map<Value*, Value*>& done)
 {
-    return (x->getFunc()->eq(y->getFunc()));
+    if (done.find(x)!=done.end()) {
+      return (done[x]==y);
+    }
+    done[x]=y; //safe since we short-circuit if anything's unequal
+    kint eq = (x->getFunc()->eq(y->getFunc(),done));
+    return eq;
 }
 
 /// This is probably quite meaningless
@@ -146,10 +177,10 @@ Value* fntable_marshal(VMState* vm,vector<Value*>& done,Value* x, int id)
     int idx;
     vector<Value*>::iterator it = done.begin();
     for(idx=0;it!=done.end();++it,++idx) {
-	if ((*it)==x) {
+	if ((*it)->getRaw()==x->getRaw()) {
 	    // Already done it, just spit out a reference to it.
 	    wchar_t buf[20];
-	    SWPRINTF(buf,20,L"[%d]C[%d]",id,idx);
+	    SWPRINTF(buf,20,L"C[%d]",idx);
 	    return new Value(new String(buf),KVT_STRING);
 	}
     }
@@ -158,14 +189,14 @@ Value* fntable_marshal(VMState* vm,vector<Value*>& done,Value* x, int id)
 
     Closure *c = x->getFunc();
     wchar_t buf[21];
-    SWPRINTF(buf,21,L"[%d]F[[%d]",id,idx);
+    SWPRINTF(buf,21,L"F[[%d]",idx);
     String* mclos = new String(buf);
     wchar_t cfnid[50];
     SWPRINTF(cfnid,50,L"[%d][%d]",c->getFnID(),c->getNumArgs());
     mclos->append(cfnid);
     Value** cargs = c->getArgs();
 
-    for(int i=0;i<c->getNumArgs();i++) {
+    for(int i=0;i<c->getNumArgs();++i) {
 	wchar_t* el = funtable_marshal_aux(vm,done,cargs[i],id);
 	mclos->append(el);
     }
@@ -175,6 +206,41 @@ Value* fntable_marshal(VMState* vm,vector<Value*>& done,Value* x, int id)
 //    return new Value(new String("<<closure>>"),stringtable);
 }
 
+KayaValue findIdentical(KayaValue v,map<KayaValue, KayaValue> done) {
+    map<KayaValue, KayaValue>::iterator it = done.begin();
+    for(;it!=done.end();++it) {
+	if ((*it).first->getRaw() == v->getRaw()) {
+	    return (*it).second;
+	}
+    }
+    return NULL;
+}
+
+Value* fntable_reflect(VMState* vm,map<KayaValue, KayaValue>& done,Value* x) {
+    // Check for circles.
+    KayaValue c = findIdentical(x,done);
+    if (c!=NULL) return c;
+
+    Closure* inc = x->getFunc();
+
+    Value* v = KayaUnion(5,2);
+    done[x] = v;
+
+    KArray args = newKayaArray(inc->getNumArgs());
+
+    Value** cargs = inc->getArgs();
+    for(int i=0;i<inc->getNumArgs();++i) {
+	Value* el = funtable_reflect_aux(vm,done,cargs[i]);
+	KayaArrayPush(args,el);
+    }
+
+    Value* uarr = KayaArrayVal(args);
+    KayaUnionSetArg(v,0,KayaInt(inc->getFnID()));
+    KayaUnionSetArg(v,1,uarr);
+
+    return v;
+}
+
 Value* fntable_hash(Value* x)
 {
     // FIXME: Do this. Not sure functions will hash sensibly.
@@ -194,7 +260,12 @@ int fntable_memusage(Value* x)
 
 /// Functions on Reals
 
-kint realtable_eq(Value* x, Value* y)
+kint realtable_eq(Value* x, Value* y, map<Value*, Value*>& done)
+{
+    return (x->getReal()==y->getReal());
+}
+
+kint realtable_fasteq(Value* x, Value* y)
 {
     return (x->getReal()==y->getReal());
 }
@@ -227,11 +298,19 @@ Value* realtable_marshal(VMState* vm,vector<Value*>& done,Value* x, int id)
 {
     Value* v = new Value(NULL,KVT_STRING);
     wchar_t buf[255];
-    SWPRINTF(buf,255,L"[%d]R[%e]",id,x->getReal());
+    SWPRINTF(buf,255,L"R[%e]",x->getReal());
     v->setString(new String(buf));
     return v;
 }
 
+Value* realtable_reflect(VMState* vm,map<KayaValue, KayaValue>& done,Value* x)
+{
+    Value* v = KayaUnion(1,1);
+    Value* f = KayaFloat(KFLOAT(x->getReal()));
+    KayaUnionSetArg(v,0,f);
+    return v;
+}
+
 Value* realtable_hash(Value* x)
 {
     vector<Value*> done;
@@ -246,9 +325,14 @@ int realtable_memusage(Value* x)
 
 /// Functions on Arrays
 
-kint arraytable_eq(Value* x, Value* y)
+kint arraytable_eq(Value* x, Value* y, map<Value*, Value*>& done)
 {
-    return (x->getArray()->eq(y->getArray()));
+    if (done.find(x)!=done.end()) {
+	return (done[x]==y);
+    }
+    done[x]=y;
+    kint eq = (x->getArray()->eq(y->getArray(),done));
+    return eq;
 }
 
 kint arraytable_cmp(Value* x, Value* y)
@@ -272,16 +356,16 @@ Value* arraytable_copy(Value* x, map<Value*,Value*>& done)
     return copied;
 }
 
-Value* arraytable_marshal(VMState* vm,vector<Value*>& done,Value* x, int id)
+Value* arraytable_marshal(VMState* vm,vector<KayaValue>& done,Value* x, int id)
 {
     // Check for circles.
     int idx;
     vector<Value*>::iterator it = done.begin();
     for(idx=0;it!=done.end();++it,++idx) {
-	if ((*it)==x) {
+	if ((*it)->getRaw()==x->getRaw()) {
 	    // Already done it, just spit out a reference to it.
 	    wchar_t buf[20];
-	    SWPRINTF(buf,20,L"[%d]C[%d]",id,idx);
+	    SWPRINTF(buf,20,L"C[%d]",idx);
 	    return new Value(new String(buf),KVT_STRING);
 	}
     }
@@ -290,9 +374,9 @@ Value* arraytable_marshal(VMState* vm,vector<Value*>& done,Value* x, int id)
 
     Array* v = x->getArray();
     wchar_t buf[34];
-    SWPRINTF(buf,34,L"[%d]A[[%d][%d]",id,idx,v->reserved_size());
+    SWPRINTF(buf,34,L"A[[%d][%d]",idx,v->reserved_size());
     String* marray = new String(buf);
-    for(int i=0;i<v->size();i++) {
+    for(int i=0;i<v->size();++i) {
 	wchar_t* el = funtable_marshal_aux(vm,done,v->lookup(i),id);
 	marray->append(el);
     }
@@ -300,6 +384,30 @@ Value* arraytable_marshal(VMState* vm,vector<Value*>& done,Value* x, int id)
     return new Value(marray,KVT_STRING);
 }
 
+Value* arraytable_reflect(VMState* vm,map<KayaValue, KayaValue>& done,Value* x)
+{
+    // Check for circles.
+    KayaValue c = findIdentical(x,done);
+    if (c!=NULL) return c;
+
+    Array* inarr = x->getArray();
+
+    Value* v = KayaUnion(3,1);
+    done[x] = v;
+
+    KArray ar = newKayaArray(inarr->size());
+
+    for(int i=0;i<inarr->size();++i) {
+	Value* el = funtable_reflect_aux(vm,done,inarr->lookup(i));
+	KayaArrayPush(ar,el);
+    }
+
+    Value* varr = KayaArrayVal(ar);
+    KayaUnionSetArg(v,0,varr);
+
+    return v;
+}
+
 Value* arraytable_hash(Value* x)
 {
     // FIXME: Do this.
@@ -316,9 +424,14 @@ int arraytable_memusage(Value* x)
 
 /// Functions on Unions
 
-kint uniontable_eq(Value* x, Value* y)
+kint uniontable_eq(Value* x, Value* y, map<Value*, Value*>& done)
 {
-    return (x->getUnion()->eq(y->getUnion()));
+    if (done.find(x)!=done.end()) {
+	return (done[x]==y);
+    }
+    done[x]=y;
+    kint eq = (x->getUnion()->eq(y->getUnion(),done));
+    return eq;
 }
 
 kint uniontable_cmp(Value* x, Value* y)
@@ -340,16 +453,16 @@ Value* uniontable_copy(Value* x, map<Value*,Value*>& done)
     return thisval;
 }
 
-Value* uniontable_marshal(VMState* vm,vector<Value*>& done,Value* x, int id)
+Value* uniontable_marshal(VMState* vm,vector<KayaValue>& done,Value* x, int id)
 {
     // Check for circles.
     int idx;
     vector<Value*>::iterator it = done.begin();
     for(idx=0;it!=done.end();++it,++idx) {
-	if ((*it)==x) {
+	if ((*it)->getRaw()==x->getRaw()) {
 	    // Already done it, just spit out a reference to it.
 	    wchar_t buf[20];
-	    SWPRINTF(buf,20,L"[%d]C[%d]",id,idx);
+	    SWPRINTF(buf,20,L"C[%d]",idx);
 	    return new Value(new String(buf),KVT_STRING);
 	}
     }
@@ -358,13 +471,13 @@ Value* uniontable_marshal(VMState* vm,vector<Value*>& done,Value* x, int id)
 
     Union *u = x->getUnion();
     wchar_t buf[21];
-    SWPRINTF(buf,21,L"[%d]U[[%d]",id,idx);
+    SWPRINTF(buf,21,L"U[[%d]",idx);
     String* munion = new String(buf);
     wchar_t utag[50];
     SWPRINTF(utag,50,L"[%d][%d]",U_TAG(u),U_ARITY(u));
     munion->append(utag);
 
-    for(int i=0;i<U_ARITY(u);i++) {
+    for(int i=0;i<U_ARITY(u);++i) {
 	wchar_t* el = funtable_marshal_aux(vm,done,u->args[i],id);
 	munion->append(el);
     }
@@ -372,6 +485,33 @@ Value* uniontable_marshal(VMState* vm,vector<Value*>& done,Value* x, int id)
     return new Value(munion,KVT_STRING);
 }
 
+
+
+Value* uniontable_reflect(VMState* vm,map<KayaValue, KayaValue>& done,Value* x)
+{
+    // Check for circles.
+    KayaValue c = findIdentical(x,done);
+    if (c!=NULL) return c;
+
+    Union* inu = x->getUnion();
+
+    Value* v = KayaUnion(4,2);
+    done[x] = v;
+
+    KArray flds = newKayaArray(U_ARITY(inu));
+
+    for(int i=0;i<U_ARITY(inu);++i) {
+	Value* el = funtable_reflect_aux(vm,done,inu->args[i]);
+	KayaArrayPush(flds,el);
+    }
+
+    Value* uarr = KayaArrayVal(flds);
+    KayaUnionSetArg(v,0,KayaInt(U_TAG(inu)));
+    KayaUnionSetArg(v,1,uarr);
+
+    return v;
+}
+
 Value* uniontable_hash(Value* x)
 {
     return 0;
@@ -386,7 +526,12 @@ int uniontable_memusage(Value* x)
 
 /// Functions on Exceptions
 
-kint exceptiontable_eq(Value* x, Value* y)
+kint exceptiontable_eq(Value* x, Value* y, map<Value*, Value*>& done)
+{
+    return (x->getExcept()->eq(y->getExcept()));
+}
+
+kint exceptiontable_fasteq(Value* x, Value* y)
 {
     return (x->getExcept()->eq(y->getExcept()));
 }
@@ -415,10 +560,16 @@ Value* exceptiontable_fastcopy(Value* x)
 
 Value* exceptiontable_marshal(VMState* vm,vector<Value*>& done,Value* x, int id)
 {
-    vm->kaya_throw(L"Can't marshal exceptions sensibly",1);
+    vm->kaya_rtsError(CANT_MARSHAL_EXCEPTIONS);
     return new Value(new String(L"<<exception>>"),KVT_STRING);
 }
 
+Value* exceptiontable_reflect(VMState* vm,map<KayaValue, KayaValue>& done,Value* x)
+{
+    vm->kaya_rtsError(CANT_REFLECT_EXCEPTIONS);
+    return NULL;
+}
+
 Value* exceptiontable_hash(Value* x)
 {
     return 0;
