@@ -25,6 +25,7 @@
 #include <sys/types.h>
 #include <sys/time.h> 
 #include <unistd.h>
+#include <cstdlib>
 #include <vector>
 //#include <malloc.h>
 
@@ -45,6 +46,9 @@
 #include "VMState.h"
 #include "KayaAPI.h"
 
+// Read lines from files this many characters at a time
+#define LINE_CHUNK 1024
+
 KayaArray args;
 ArgMap httpargs;
 
@@ -77,60 +81,120 @@ int strtoint(wchar_t* str, int base)
 
 wchar_t* getsubstr(wchar_t* x,int i,int len)
 {
-//    int max = strlen(x);
     wchar_t* n = (wchar_t*)GC_MALLOC_ATOMIC((len+1)*sizeof(wchar_t));
-    wcsncpy(n,x+i,len);
-    n[len] = '\0';
-/*    for(int j=0;j<len && (j+i)<max;j++) {
-	n[j] = x[j+i];
+    if (len > 0) {
+        wcsncpy(n,x+i,len);
     }
-    n[len]='\0';*/
+    n[len] = '\0';
     return n;
 }
 
+void str_offset(Value* str, int inc)
+{
+    str->getString()->offset(inc);
+}
+
+void str_chop(Value* str, int inc)
+{
+    str->getString()->chop(inc);
+}
+
 wchar_t* getstrend(wchar_t* x,int i)
 {
     int max = wcslen(x);
     wchar_t* n = (wchar_t*)GC_MALLOC_ATOMIC((max+1-i)*sizeof(wchar_t));
-    wcsncpy(n,x+i,max-i);
-    n[max-i] = '\0';
-/*    for(int j=0;j<len && (j+i)<max;j++) {
-	n[j] = x[j+i];
+    if (max-i > 0) {
+      wcsncpy(n,x+i,max-i);
     }
-    n[len]='\0';*/
+    n[max-i] = '\0';
     return n;
 }
-
-wchar_t* readstr()
-{
-    char* buf=(char*)GC_MALLOC_ATOMIC(255*sizeof(char));
-    fgets(buf,255,stdin);
-    char *loc = strchr(buf,'\n');
-    *loc = '\0';
-    return strtowc(buf);
-}
-
 wchar_t getIndex(wchar_t* str,int i)
 {
     return str[i];
 }
 
+void setIndex(wchar_t* str, wchar_t c,int i)
+{
+    str[i] = c;
+}
+
 wchar_t* getLine(FILE* f)
 {
-// I know there's a better way. Can't be bothered.
-    char* buf=(char*)GC_MALLOC_ATOMIC(4096*sizeof(char)); 
-    fgets(buf,4096,f);
-    return strtowc(buf);    
+    unsigned size = LINE_CHUNK; // #define at top of file
+    // buffer to read into
+    char* buf=(char*)GC_MALLOC_ATOMIC(LINE_CHUNK*sizeof(char)+1); 
+    // accumulator for result; but only allocate it when it turns out to be
+    // needed (i.e., when one chunk isn't enough).
+    char* res=NULL;
+    char* r;
+    do {
+	r = fgets(buf,LINE_CHUNK,f);
+	if (r == NULL) { // End of file, or read error. FIXME: Check which.
+	    if (res==NULL)
+	    {
+		// Fail silently (should this throw an exception?)
+		return emptyString(); 
+	    } else {
+		// File finished without a newline, return what we've got.
+		return strtowc(res);
+	    }
+	}
+	if (res==NULL) {
+	    // FIXME: We're relying on \n or \r\n line endings. How much
+	    // can we rely on this?
+	    // C defines \n to be the native newline when reading
+	    // or writing files in text mode so maybe we're okay...
+	    if (strlen(buf)<(size-1) || buf[LINE_CHUNK-2]=='\n') {
+		return strtowc(buf); // Success in one chunk
+	    }
+	    // Begin allocation for accumulator and carry on as normal.
+	    size += LINE_CHUNK-1;
+	    res = (char*)GC_MALLOC_ATOMIC(size*sizeof(char)+1);
+	    strcpy(res,buf);
+	} 
+	else {
+	    strcat(res,buf);
+	    if (strlen(res)<(size-1) || buf[LINE_CHUNK-2]=='\n') { 
+                // Finished successfully
+		return strtowc(res); 
+	    } else {
+		size += LINE_CHUNK-1; // -1 to account for null termination of
+		                      // string read into buf
+		res = (char*)GC_REALLOC(res, size*sizeof(char)+1);
+	    }
+	}
+    } while (r!=NULL);
+    return emptyString(); // Failed, although we shouldn't get here
 }
 
 wchar_t* getString(FILE* f)
 {
-// I know there's a better way. Can't be bothered.
-    char* buf=(char*)GC_MALLOC_ATOMIC(4096*sizeof(char)); 
+    unsigned size = LINE_CHUNK; // #define at top of file
+    // buffer to read into
+    char* buf=(char*)GC_MALLOC_ATOMIC(LINE_CHUNK*sizeof(char)+1); 
+    // accumulator for result; but only allocate it when it turns out to be
+    // needed (i.e., when one chunk isn't enough).
     int i=0;
-    while((buf[i++]=fgetc(f))!='\0') {}
-    buf[i]='\0';
-    return strtowc(buf);    
+    do {
+      
+      do {
+	int c = fgetc(f);
+	if (c == EOF) {
+	  buf[i] = '\0';
+	  return strtowc(buf);
+	} 
+	buf[i] = (unsigned char)c;
+	if (buf[i] == '\0') {
+	  return strtowc(buf);
+	}
+	i++;
+      } while (i%LINE_CHUNK != 0);
+      size += LINE_CHUNK;
+      buf = (char*)GC_REALLOC(buf, size*sizeof(char)+1);
+
+    } while (buf[i] != EOF);
+    return emptyString(); // Failed, although we shouldn't get here
 }
 
 void putLine(FILE* f, wchar_t* c)
@@ -161,9 +225,26 @@ void putStr(wchar_t* rawc)
     cout << c;
 }
 
+// Check if the top bit is set in a string (i.e., whether it is ASCII or
+// includes UTF-8 characters)
+int topSet(const char* str) {
+    const char* sptr = str;
+    for(;(*sptr)!=0;++sptr) {
+	if (((*sptr)&128)!=0) {
+	    return 1;
+	}
+    }
+    return 0;
+}
+
+int checkUTF8(wchar_t* str)
+{
+    return(topSet(CSTRING(str)));
+}
+
 void printArray(KayaArray foo)
 {
-    for(int i=0;i<foo->size();i++) {
+    for(int i=0;i<foo->size();++i) {
 	cout << wctostr(foo->lookup(i)->getString()->getVal()) << endl;
     }
 }
@@ -235,7 +316,7 @@ bool validFile(FILE* f)
 void storeArgs(int argc,char* argv[])
 {
     args = new Array(argc);
-    for (int i=0;i<argc;i++) {
+    for (int i=0;i<argc;++i) {
 	KayaValue arg = new Value((void*)(new String(strtowc(argv[i]))),KVT_STRING);
 	args->push_back(arg);
     }
@@ -358,7 +439,7 @@ int runProgram(wchar_t* prog, KayaArray args)
     char** argv = new char*[args->size()+2];
 
     argv[0]=wctostr(prog);
-    for(int i=0;i<args->size();i++) {
+    for(int i=0;i<args->size();++i) {
 	argv[i+1]=wctostr(args->lookup(i)->getString()->getVal());
     }
     argv[args->size()+1]=NULL;
@@ -417,19 +498,12 @@ int do_wcscspn(wchar_t c, wchar_t* s) {
   buf[0] = c;
   buf[1] = '\0';
   return wcscspn(s,buf);
-  /*  int i = 0;
-  while (s[i] != 0) {
-    if (s[i] == c) {
-      return i;
-    }
-    i++;
-  }
-  return i; */
 }
 
 /** The following base64 functions are adapted from
  * http://base64.sourceforge.net/b64.c
  * By Bob Trower, Copyright (c) Trantor Standard Systems Inc., 2001
+ * See COPYING.b64 for redistribution conditions
  */
 // Modified by CIM to work with wchar_t input
 /*
@@ -480,19 +554,19 @@ char* b64enc(char* block, int len) {
     x=0; // Input pointer
     y=0; // Output pointer
     while(x<len) {
-	for(i = 0; i<3;i++) {
+	for(i = 0; i<3;++i) {
 	    if (x>=len) { 
 		in[i]=0;
 	    } else {
 		unsigned char v = (unsigned char)block[x];
 		in[i]=v;
 	    }
-	    x++;
+	    ++x;
 	}
 	encodeblock(in,out,3);
-	for(i=0;i<4;i++) {
+	for(i=0;i<4;++i) {
 	    outblock[y] = out[i];
-	    y++;
+	    ++y;
 	}
     }
     outblock[y]='\0';
@@ -512,7 +586,7 @@ char* b64dec(char* block, KayaValue outlen) {
     x=0; // Input pointer
     y=0; // Output pointer
     while(x<len) {
-	for(i = 0; i<4;i++) {
+	for(i = 0; i<4;++i) {
 	    if (x>=len) { 
 		in[i]=0;
 	    } else {
@@ -523,29 +597,21 @@ char* b64dec(char* block, KayaValue outlen) {
                 }
 		in[i]=v-1;
 	    }
-	    x++;
+	    ++x;
 	}
 	decodeblock(in,out);
-	for(i=0;i<3;i++) {
+	for(i=0;i<3;++i) {
 	    outblock[y] = out[i];
-	    y++;
+	    ++y;
 	}
     }
     outblock[y]='\0';
-    KayaSetInt(outlen,y-1);
+    KayaSetInt(outlen,y);
     return outblock;
 }
 
 /** End of Bob Trower's code */
 
-
-/*
-int funtable_compare(KayaValuex, KayaValue y)
-{
-    return x->getFunTable()->cmp(x,y)->getInt();
-}
-*/
-
 typedef enum { GENERAL, INTQS, STRQS } QSver;
 
 void qs(VMState* vm, KayaArray xs,int l, int r,Closure* f, QSver version);
@@ -593,7 +659,7 @@ int partition(VMState* vm, KayaArray xs, int l, int r, int pi,
     int si = l;
     int res;
 
-    for(int i=l; i<r; i++) {
+    for(int i=l; i<r; ++i) {
 	xsi = xs->lookup(i);
 	xssi = xs->lookup(si);
 
@@ -617,7 +683,7 @@ int partition(VMState* vm, KayaArray xs, int l, int r, int pi,
 	    xssi->setPtr(xsi);
 	    xsi->setPtr(tmp);
 //	    swap(xs->lookup(si),xs->lookup(i));
-	    si++;
+	    ++si;
 	}
     }
 //    swap(xs->lookup(r),xs->lookup(si));
@@ -643,7 +709,7 @@ int funtable_compare(void* vmptr,KayaValue x,KayaValue y)
   // NULL pointer for vmptr means something else is confident x and y exist
     if (vmptr != NULL && (x == NULL || x->getType() == KVT_NULL || y == NULL || y->getType() == KVT_NULL)) {
 	VMState* vm = (VMState*)vmptr;
-	vm->kaya_throw("Tried to compare a non-existent value",1);
+	vm->kaya_rtsError(INVALID_VALUE);
     }
 #endif
     int r = 0;
@@ -656,12 +722,58 @@ bool funtable_equal(void* vmptr,KayaValue x, KayaValue y)
 #ifndef NOCHECK
     if (vmptr != NULL && (x == NULL || x->getType() == KVT_NULL || y == NULL || y->getType() == KVT_NULL)) {
 	VMState* vm = (VMState*)vmptr;
-	vm->kaya_throw("Tried to compare a non-existent value",1);
+	vm->kaya_rtsError(INVALID_VALUE);
     }
 #endif
+    valtype vt = x->getType();
+    switch (vt) {
+    case KVT_INT:
+      return inttable_fasteq(x,y);
+    case KVT_STRING:
+      return stringtable_fasteq(x,y);
+    case KVT_REAL:
+      return realtable_fasteq(x,y);
+    case KVT_EXCEPTION:
+      return exceptiontable_fasteq(x,y);
+    default:
+      map<Value*, Value*> done;
+      return funtable_equal_aux(x,y,done);
+    }
+}
+
+bool funtable_equal_aux(KayaValue x, KayaValue y, map<Value*, Value*>& done)
+{
     bool r = 0;
-    FUNTABLE_OP(eq,x,y)
-	return r;
+    if (x->getFunTable() != y->getFunTable()) {
+      return 0;
+    }
+    switch (x->getFunTable()) { 
+    case KVT_INT: 
+	r = inttable_eq(x,y,done); 
+	break; 
+    case KVT_STRING: 
+	r = stringtable_eq(x,y,done); 
+	break; 
+    case KVT_FUNC: 
+	r = fntable_eq(x,y,done); 
+	break; 
+    case KVT_ARRAY: 
+	r = arraytable_eq(x,y,done); 
+	break; 
+    case KVT_UNION: 
+	r = uniontable_eq(x,y,done); 
+	break; 
+    case KVT_EXCEPTION: 
+	r = exceptiontable_eq(x,y,done); 
+	break; 
+    case KVT_REAL: 
+	r = realtable_eq(x,y,done); 
+	break; 
+    case KVT_NULL: 
+	r = inttable_eq(x,y,done);
+	break; // can't happen
+    } 
+    return r;
 }
 
 
@@ -734,11 +846,16 @@ wchar_t* funtable_marshal(void* vmptr,KayaValue x, int i)
 #ifndef NOCHECK
     if (vmptr != NULL && (x == NULL || x->getType() == KVT_NULL)) {
 	VMState* vm = (VMState*)vmptr;
-	vm->kaya_throw("Tried to compare a non-existent value",1);
+	vm->kaya_rtsError(INVALID_VALUE);
     }
 #endif
     vector<KayaValue> done;
-    return funtable_marshal_aux(vmptr,done,x,i);
+    wchar_t* mdata = funtable_marshal_aux(vmptr,done,x,i);
+    wchar_t mid[27]; // can be negative
+    SWPRINTF(mid,27,L"[%d][%d]",i,getFnHash());
+    String* idmdata = new String(mid);
+    idmdata->append(mdata);
+    return idmdata->getVal();
 }
 
 wchar_t* funtable_marshal_aux(void* vmptr,vector<KayaValue>& done,KayaValue x, int i)
@@ -774,12 +891,84 @@ wchar_t* funtable_marshal_aux(void* vmptr,vector<KayaValue>& done,KayaValue x, i
     return r->getString()->getVal();
 }
 
+Value* funtable_reflect(void* vmptr,KayaValue x)
+{
+#ifndef NOCHECK
+    if (vmptr != NULL && (x == NULL || x->getType() == KVT_NULL)) {
+	VMState* vm = (VMState*)vmptr;
+	vm->kaya_rtsError(INVALID_VALUE);
+    }
+#endif
+    map<KayaValue, KayaValue> done;
+    return funtable_reflect_aux(vmptr,done,x);
+}
+
+Value* funtable_reflect_aux(void* vmptr,
+			    map<KayaValue, KayaValue>& done,KayaValue x)
+{
+    VMState* vm = (VMState*)vmptr;
+    Value* r = new Value(NULL,KVT_NULL);
+    switch (x->getFunTable()) { 
+    case KVT_INT: 
+      r = inttable_reflect(vm,done,x);
+      break;
+    case KVT_STRING: 
+      r = stringtable_reflect(vm,done,x);
+      break;
+    case KVT_FUNC: 
+      r = fntable_reflect(vm,done,x);
+      break;
+    case KVT_ARRAY: 
+      r = arraytable_reflect(vm,done,x); 
+      break;
+    case KVT_UNION: 
+      r = uniontable_reflect(vm,done,x);
+      break;
+    case KVT_EXCEPTION: 
+      r = exceptiontable_reflect(vm,done,x);
+      break;
+    case KVT_REAL: 
+      r = realtable_reflect(vm,done,x); 
+      break;
+    case KVT_NULL:
+      r = inttable_reflect(vm,done,x);
+      break; //can't happen
+    } 
+    return r;
+}
+
+KayaValue reifyUnion(int tag, int arity)
+{
+    return KayaUnion(tag, arity);
+}
+
+void reifyUnionArgs(KayaValue v, KayaArray flds)
+{
+    for(int i=0;i<flds->size();++i) {
+	KayaUnionSetArg(v,i,KayaArrayLookup(flds,i));
+    }
+}
+
+KayaValue reifyClosure(int fnid, int arity)
+{
+    Closure* c = new Closure(NULL, getFn(fnid), arity, arity, false);
+    return new Value((void*)c,KVT_FUNC);
+}
+
+void reifyClosureArgs(KayaValue v, KayaArray flds)
+{
+    Closure* c = v->getFunc();
+    for(int i=0;i<flds->size();++i) {
+	c->setArg(i,KayaArrayLookup(flds,i));
+    }
+}
+
 KayaValue funtable_copy(void* vmptr,KayaValue x)
 {
 #ifndef NOCHECK
   if (x == NULL || x->getType() == KVT_NULL) {
     VMState* vm = (VMState*)vmptr;
-    vm->kaya_throw("Tried to copy a non-existent value",1);
+    vm->kaya_rtsError(INVALID_VALUE);
   }
 #endif
     valtype vt = x->getType();
@@ -852,7 +1041,7 @@ void callFnID(void* vmptr, int id, KayaValue arg)
 	PUSH(arg);
 	f(vm);
     } else {
-	vm->kaya_throw("Illegal function id",1);
+	vm->kaya_rtsError(UNKNOWN_FUNCTION_ID);
     }
 }
 #ifndef WIN32
@@ -992,6 +1181,18 @@ int do_unlink(const wchar_t* fn) {
   return unlink(wctostr(fn));
 }
 
+wchar_t* dogetcwd() {
+  unsigned int sz = 1024;
+  char* buf;
+  char* rv;
+  do {
+    buf = (char*)GC_MALLOC_ATOMIC(sz*sizeof(char));
+    rv = getcwd(buf,sz);
+    sz = sz*2;
+  } while (rv == NULL);
+  return KSTRING(buf);
+}
+
 int do_mkdir(const wchar_t* fn, int um) {
 #ifdef WIN32
 	return mkdir(wctostr(fn));
@@ -1113,7 +1314,7 @@ KayaValue do_stat(void* vmptr, wchar_t* name)
     struct stat* buf = (struct stat*)KayaAlloc(sizeof(struct stat));
     int ret = stat(wctostr(name), buf);
     if (ret==-1) {
-      	vm->kaya_throw(strerror(errno),errno);
+      	vm->kaya_internalError(errno);
     }
 
     KayaValue val = KayaUnion(0,5);
@@ -1140,13 +1341,13 @@ wchar_t* do_urlencode(void* vm, wchar_t* strinwide)
     char* strout = (char*)KayaAlloc(strlen(strin)*3+1);
     if (strout==NULL) {
 	// something broke, probably duff input.
-	vmptr->kaya_throw("urlencode error",1);
+	vmptr->kaya_internalError(1);
     }
     char* r = strout;
     while(*strin!='\0') {
 	if (*strin==' ') {
 	    *strout = '+';
-	    strout++;
+	    ++strout;
 	}
 	else if (ispunct(*strin) || *strin<32)
 	{
@@ -1157,9 +1358,9 @@ wchar_t* do_urlencode(void* vm, wchar_t* strinwide)
 	}
 	else {
 	    *strout = *strin;
-	    strout++;
+	    ++strout;
 	}
-	strin++;
+	++strin;
     }
     *strout='\0';
     return KSTRING(r);
@@ -1172,7 +1373,7 @@ wchar_t* do_urldecode(void* vm, wchar_t* strinwide)
     char* strout = (char*)KayaAlloc(strlen(strin)+1);
     if (strout==NULL) {
 	// something broke, probably duff input.
-	vmptr->kaya_throw("urldecode error",1);
+	vmptr->kaya_internalError(1);
     }
     char* r = strout;
     char buf[3];
@@ -1182,61 +1383,74 @@ wchar_t* do_urldecode(void* vm, wchar_t* strinwide)
     while(*strin!='\0') {
 	if (*strin=='+') {
 	    *strout = ' ';
-	    strout++;
+	    ++strout;
 	}
 	else if (*strin == '%')
 	{
 	    if (strlen(strin)<3) {
-		vmptr->kaya_throw("urldecode error",1);
+		vmptr->kaya_internalError(1);
 	    }
 	    buf[0] = strin[1];
 	    buf[1] = strin[2];
 	    *strout = (char)strtol(buf,&convcheck,16);
 	    if (*convcheck != '\0') {
-		vmptr->kaya_throw("urldecode error",1);
+		vmptr->kaya_internalError(1);
 	    }
 	    strin+=2;
-	    strout++;
+	    ++strout;
 	}
 	else {
 	    *strout = *strin;
-	    strout++;
+	    ++strout;
 	}
-	strin++;
+	++strin;
     }
     *strout='\0';
     return KSTRING(r);
 }
 
 FILE *do_fopen(const wchar_t* path, const wchar_t* mode) {
-  return fopen(wctostr(path),wctostr(mode));
+    return fopen(wctostr(path),wctostr(mode));
 }
 
 FILE *do_freopen(const wchar_t* path, const wchar_t* mode, FILE* stream) {
-  return freopen(wctostr(path),wctostr(mode),stream);
+    return freopen(wctostr(path),wctostr(mode),stream);
 }
 
 void do_rename(const wchar_t* oldfile, const wchar_t* newfile) {
-  rename(wctostr(oldfile),wctostr(newfile));
+    rename(wctostr(oldfile),wctostr(newfile));
 }
 
 void do_chdir(const wchar_t* newdir) {
-  chdir(wctostr(newdir));
+    chdir(wctostr(newdir));
 }
 
 wchar_t* do_getenv(const wchar_t* env) {
-  return strtowc(getenv(wctostr(env)));
+    return strtowc(getenv(wctostr(env)));
 }
 
 #ifndef WIN32
 int do_system(const wchar_t* prog) {
-  return system(wctostr(prog));
+    return system(wctostr(prog));
 }
+
+FILE* do_popen(const wchar_t* prog, int read) {
+    if (read) {
+	return popen(wctostr(prog),"r");
+    } else {
+	return popen(wctostr(prog),"w");
+    }
+}
+
+int do_pclose(FILE* stream) {
+    return pclose(stream);
+}
+
 #endif
 
 kint binary_length(const wchar_t* str) {
-  char* rawstr = CSTRING(str);
-  return (kint)strlen(rawstr);
+    char* rawstr = CSTRING(str);
+    return (kint)strlen(rawstr);
 }
 
 int nextPower(int x)
@@ -1284,8 +1498,13 @@ int strHash(wchar_t* m)
 {
     int hash = 0;
     int len = wcslen(m);
-    for(int i=0;i<len;i++) {
+    for(int i=0;i<len;++i) {
 	hash = 131*hash+m[i];
     }
     return hash;
 }
+
+int rtchecks()
+{
+    return RTCHECKS;
+}
