@@ -29,10 +29,14 @@ wchar_t* inttostr(int s, int base);
 int strtoint(wchar_t* str, int base);
 
 wchar_t* getsubstr(wchar_t* x,int i,int len);
+void str_offset(Value* str, int inc);
+void str_chop(Value* str, int inc);
 wchar_t* getstrend(wchar_t* x,int i);
 wchar_t* readstr();
 
 wchar_t getIndex(wchar_t* str,int i);
+void setIndex(wchar_t* str, wchar_t c,int i);
+
 wchar_t* do_urlencode(void* vm, wchar_t* strinwide);
 wchar_t* do_urldecode(void* vm, wchar_t* strinwide);
 
@@ -44,7 +48,9 @@ void do_fseek(FILE* f, kint p);
 kint do_ftell(FILE* f);
 
 void putStr(wchar_t* c);
-
+// Check if a string has any UTF8 characters in it (return non-zero if so)
+int topSet(const char* str);
+int checkUTF8(wchar_t* str);
 void putU8Str(wchar_t* c);
 
 FILE* getstdin();
@@ -127,16 +133,26 @@ switch (x->getFunTable()) { \
 
 // #define funtable_compare(x,y) x->getFunTable()->cmp(x,y)
 #define funtable_eq(v,x,y) (x==y || funtable_equal(v,x,y))
+#define funtable_eq_aux(x,y,done) (x==y || funtable_equal_aux(x,y,done))
 
 int funtable_compare(void* vmptr, KayaValue x, KayaValue y);
 bool funtable_equal(void* vmptr, KayaValue x, KayaValue y);
+bool funtable_equal_aux(KayaValue x, KayaValue y, map<Value*,Value*>& done);
 int funtable_hash(KayaValue x);
 int funtable_memusage(KayaValue x);
 wchar_t* funtable_marshal(void* vmptr,KayaValue x,int id);
 wchar_t* funtable_marshal_aux(void* vmptr,vector<KayaValue>& done,KayaValue x, int i);
+Value* funtable_reflect(void* vmptr,KayaValue x);
+Value* funtable_reflect_aux(void* vmptr,
+			    map<KayaValue, KayaValue>& done,KayaValue x);
 KayaValue funtable_copy(void* vmptr, KayaValue x);
 KayaValue funtable_copy_aux(KayaValue x, map<Value*, Value*>& done);
 
+KayaValue reifyUnion(int tag, int arity);
+void reifyUnionArgs(KayaValue v, KayaArray flds);
+KayaValue reifyClosure(int tag, int arity);
+void reifyClosureArgs(KayaValue v, KayaArray flds);
+
 bool is_initialised(KayaValue x);
 KayaValue unsafe_id(KayaValue x);
 
@@ -159,6 +175,7 @@ int maxMemUsage(void* vmptr);
 int do_access(wchar_t* pathname, KayaValue mode);
 // Added by Chris Morris, 5/9/05
 int do_unlink(const wchar_t* fn);
+wchar_t* dogetcwd();
 int do_mkdir(const wchar_t* fn, int um);
 
 int do_setrlimit(KayaValue lim, int soft, int hard);
@@ -183,6 +200,8 @@ kint binary_length(const wchar_t* str);
 
 #ifndef WIN32
 int do_system(const wchar_t* prog);
+FILE* do_popen(const wchar_t* prog, int read);
+int do_pclose(FILE* stream);
 #endif
 
 // Return next biggest power of 2.
@@ -198,4 +217,6 @@ void do_usleep(int len);
 
 int strHash(wchar_t* m);
 
+int rtchecks();
+
 #endif
