@@ -24,6 +24,17 @@ Value* unmarshal(void* vmptr,wchar_t* x, int id)
 {
     VMState* vm = (VMState*)vmptr;
     vector<Value*> done;
+    int i = read_int(vm,x);
+    if (i!=id) {
+	vm->kaya_rtsError(INVALID_MARSHALLING_ID);
+    }
+    int h = read_int(vm,x);
+    if (h!=getFnHash()) {
+/*	cout << "marshal hash: " << h << endl;
+	cout << "ftable hash: " << getFnHash() << endl; */
+	vm->kaya_rtsError(INVALID_FUNCTION_TABLE_HASH);
+    }
+
     Value* v = unpickle(vm, done, x, id);
     return v;
 }
@@ -32,70 +43,54 @@ Value* unpickle(VMState* vm, vector<Value*>& done, wchar_t*& x, int id)
 {
     Value *v;
 
-    int i = read_int(vm,x);
-    if (i!=id) {
-	vm->kaya_throw("Invalid unmarshalling id",253);
-    }
-
     switch(x[0]) {
     case 'I':
-	x++;
-	v=un_int(vm,done,x);
+	v=un_int(vm,done,++x);
 	return v;
     case 'S':
-	x++;
-	v=un_string(vm,done,x);
+	v=un_string(vm,done,++x);
 	return v;
     case 'U':
-	x++;
-	v=un_union(vm,done,x,id);
+	v=un_union(vm,done,++x,id);
 	return v;
     case 'F':
-	x++;
-	v=un_closure(vm,done,x,id);
+	v=un_closure(vm,done,++x,id);
 	return v;
     case 'A':
-	x++;
-	v=un_array(vm,done,x,id);
+	v=un_array(vm,done,++x,id);
 	return v;
     case 'R':
-	x++;
-	v=un_real(vm,done,x);
+	v=un_real(vm,done,++x);
 	return v;
     case 'C':
-	x++;
-	v=un_circle(vm,done,x);
+	v=un_circle(vm,done,++x);
 	return v;
     default:
-	vm->kaya_throw("Invalid value to unmarshall",254);
+	vm->kaya_rtsError(INVALID_VALUE);
 	return new Value(NULL,KVT_NULL);
     }
 }
 
 int read_int(VMState* vm, wchar_t* &x) {
     wchar_t buf[30]; // = (wchar_t*)GC_MALLOC_ATOMIC(sizeof(wchar_t)*20);
-    int i=0;
+    int i=-1;
     
     if (*x!='[') {
-	String *err = new String(L"Invalid int to unmarshall : ");
-	err->append(new String(x));
-	vm->kaya_throw(err->getVal(),254);
+	vm->kaya_rtsError(INVALID_VALUE);
     }
-    x++; // Move past the initial [
+    ++x; // Move past the initial [
     while(*x!=']' && i<30) {
-	buf[i++]=*x;
-	x++;
+	buf[++i]=*x;
+	++x;
     }
     if (i == 50) {
-	String *err = new String(L"Invalid int to unmarshall : ");
-	err->append(new String(x));
-	vm->kaya_throw(err->getVal(),254);
+	vm->kaya_rtsError(INVALID_VALUE);
     }
-    buf[i]='\0';
+    buf[++i]='\0';
     int val = wcstoimax(buf,NULL,10);
     //      = atoi(wctostr(buf));
 
-    x++;
+    ++x;
     return val;
 }
 
@@ -109,35 +104,32 @@ Value* un_real(VMState* vm,vector<Value*>& done,wchar_t* &x)
 {
     // Expecting [num]
     wchar_t buf[100]; // = (wchar_t*)GC_MALLOC_ATOMIC(sizeof(wchar_t)*100); // Yeah, horrible, I know.
-    int i=0;
+    int i=-1;
     
     if (*x!='[') {
-	String *err = new String(L"Invalid float to unmarshall : ");
-	err->append(new String(x));
-	vm->kaya_throw(err->getVal(),254);
+	vm->kaya_rtsError(INVALID_VALUE);
     }
-    x++; // Move past the initial [
+    ++x; // Move past the initial [
     while(*x!=']' && i<100) {
-	buf[i++]=*x;
-	x++;
+	buf[++i]=*x;
+	++x;
     }
-    buf[i]='\0';
+    buf[++i]='\0';
     double val = wcstod(buf,NULL);
 
-    x++;
+    ++x;
     return MKREAL(val);
-//    vm->kaya_throw("Not implemented yet",255);
-//    return new Value(NULL,NULL);
 }
 
 Value* un_circle(VMState* vm,vector<Value*>& done,wchar_t* &x)
 {
-    if (*x!='[') vm->kaya_throw("Invalid circular structure",254);
+    if (*x!='[') vm->kaya_rtsError(INVALID_CIRCULAR_STRUCTURE);
     int circid = read_int(vm,x);
 //    cout << circid << "," << done.size() << endl;
     if (done.size()<(unsigned)circid) 
-	vm->kaya_throw("Invalid circular structure",253);
-    return done[circid];
+	vm->kaya_rtsError(INVALID_CIRCULAR_STRUCTURE);
+    // clone or we get some very odd effects - CIM
+    return done[circid]->clone();
 }
 
 Value* un_string(VMState* vm,vector<Value*>& done,wchar_t* &x)
@@ -145,8 +137,8 @@ Value* un_string(VMState* vm,vector<Value*>& done,wchar_t* &x)
     // Expecting [num]string
 //    cout << "Doing string, x is " << x << endl;
 
-    if (*x!='[') vm->kaya_throw("Invalid string to unmarshall",254);
-    x++;
+    if (*x!='[') vm->kaya_rtsError(INVALID_VALUE);
+    ++x;
     int len = read_int(vm,x);
     int alloc = read_int(vm,x);
     if (alloc < len) {
@@ -158,10 +150,10 @@ Value* un_string(VMState* vm,vector<Value*>& done,wchar_t* &x)
 
     while(i<len) {
 	buf[i++]=*x;
-	x++;
+	++x;
     }
     buf[i]='\0';
-    x++;
+    ++x;
 
 //    cout << "Got string of length " << len << "," << buf << endl;
 //    cout << "x is " << x << endl;
@@ -174,8 +166,8 @@ Value* un_array(VMState* vm,vector<Value*>& done,wchar_t* &x, int id)
 {
 //    cout << "Doing array " << endl;
 
-    if (*x!='[') vm->kaya_throw("Invalid array to unmarshall",254);
-    x++;
+    if (*x!='[') vm->kaya_rtsError(INVALID_VALUE);
+    ++x;
     int circid = read_int(vm,x);
     int sz = read_int(vm,x); // advisory only, but it makes memory usage much more efficient if it's right...
     if (sz == 0) { // unless it's zero, in which case explictly set a small size
@@ -187,7 +179,7 @@ Value* un_array(VMState* vm,vector<Value*>& done,wchar_t* &x, int id)
     // Remember the value in case we get a circular structure
     done.push_back(thisval);
     if ((unsigned)circid!=done.size()-1) {
-	vm->kaya_throw("ECB made a flawed assumption. Shout at him.",252);
+	vm->kaya_internalError(252);
     }
     
     while(x[0]!=']')
@@ -195,7 +187,7 @@ Value* un_array(VMState* vm,vector<Value*>& done,wchar_t* &x, int id)
 	Value* v = unpickle(vm,done,x,id);
 	ar->push_back(v);
     }
-    x++;
+    ++x;
 //    thisval->setPtr(new Value(ar,arraytable));
     return thisval;
 }
@@ -204,8 +196,8 @@ Value* un_union(VMState* vm,vector<Value*>& done,wchar_t* &x, int id)
 {
 //    cout << "Doing union " << endl;
 
-    if (*x!='[') vm->kaya_throw("Invalid union to unmarshall",254);
-    x++;
+    if (*x!='[') vm->kaya_rtsError(INVALID_VALUE);
+    ++x;
 
     int circid = read_int(vm,x);
     int tag = read_int(vm,x);
@@ -222,7 +214,7 @@ Value* un_union(VMState* vm,vector<Value*>& done,wchar_t* &x, int id)
     // Remember the value in case we get a circular structure
     done.push_back(thisval);
     if ((unsigned)circid!=done.size()-1) {
-	vm->kaya_throw("ECB made a flawed assumption. Shout at him.",252);	
+	vm->kaya_internalError(252);	
     }
 
 //    cout << "Adding " << circid << "," << done.size() << endl;
@@ -232,9 +224,9 @@ Value* un_union(VMState* vm,vector<Value*>& done,wchar_t* &x, int id)
     {
 	Value* v = unpickle(vm,done,x,id);
 	u->args[i]=v;
-	i++;
+	++i;
     }
-    x++;
+    ++x;
 
     return thisval;
 }
@@ -243,8 +235,8 @@ Value* un_closure(VMState* vm,vector<Value*>& done,wchar_t* &x, int id)
 {
 //    cout << "Doing union " << endl;
 
-    if (*x!='[') vm->kaya_throw("Invalid closure to unmarshall",254);
-    x++;
+    if (*x!='[') vm->kaya_rtsError(INVALID_VALUE);
+    ++x;
 
     int circid = read_int(vm,x);
     int fnid = read_int(vm,x);
@@ -256,7 +248,7 @@ Value* un_closure(VMState* vm,vector<Value*>& done,wchar_t* &x, int id)
     // Remember the value in case we get a circular structure
     done.push_back(thisval);
     if ((unsigned)circid!=done.size()-1) {
-	vm->kaya_throw("ECB made a flawed assumption. Shout at him.",252);	
+	vm->kaya_internalError(252);	
     }
 
 //    cout << "Adding " << circid << "," << done.size() << endl;
@@ -266,9 +258,9 @@ Value* un_closure(VMState* vm,vector<Value*>& done,wchar_t* &x, int id)
     {
 	Value* v = unpickle(vm,done,x,id);
 	c->setArg(i,v);
-	i++;
+	++i;
     }
-    x++;
+    ++x;
 
     return thisval;
 }
