@@ -28,8 +28,8 @@ void pokestring(void* block, int offset, wchar_t* str)
 wchar_t* peekstring(void* block, int offset)
 {
     char* str = (char*)(((BYTE*)block)+offset);
-    char* buf = (char*)(KayaAlloc(strlen(str)));
-    strcpy(buf,str);
+    //    char* buf = (char*)(KayaAlloc(strlen(str)));
+    //    strcpy(buf,str);
     return strtowc(str);
 }
 
