@@ -26,7 +26,7 @@ void net_init(void* vmptr) {
   err = WSAStartup( wVersionRequested, &wsaData );
   if ( err != 0 ) {
     VMState* vm = (VMState*)vmptr;
-    vm->kaya_throw("Couldn't initialise network library",1);
+    vm->kaya_internalError(7);
   }
 #endif
 }
@@ -38,7 +38,6 @@ void* net_connect(void* vmptr, int proto,wchar_t* rawserver, int port)
     VMState* vm = (VMState*)vmptr;
     int sock;
     struct sockaddr_in addr;
-
     switch(proto) {
     case 1: 
 	sock=socket(PF_INET,SOCK_STREAM,0);
@@ -49,7 +48,7 @@ void* net_connect(void* vmptr, int proto,wchar_t* rawserver, int port)
 //	addr.sin_family=AF_INET6;
 //	break;
     default:
-	vm->kaya_throw("Protocol not supported (yet)",1);
+	vm->kaya_internalError(8);
     }
 
     struct hostent * hostinfo;
@@ -57,9 +56,9 @@ void* net_connect(void* vmptr, int proto,wchar_t* rawserver, int port)
     if (hostinfo==NULL)
     {
 #ifdef WIN32
-      vm->kaya_throw("Can't get host address!",WSAGetLastError());
+	vm->kaya_internalError(9); // ,WSAGetLastError());
 #else
-      vm->kaya_throw("Can't get host address!",1);
+	vm->kaya_internalError(9);
 #endif
     }
     addr.sin_addr=*(struct in_addr *)hostinfo->h_addr;
@@ -67,7 +66,7 @@ void* net_connect(void* vmptr, int proto,wchar_t* rawserver, int port)
 
     if (connect(sock,(struct sockaddr *)(&addr),sizeof(struct sockaddr_in))==-1)
     {
-	vm->kaya_throw("Error in connecting",1);
+	vm->kaya_internalError(10);
     }
 
     ConnInfo *c = new ConnInfo();
@@ -97,7 +96,7 @@ int net_listen(void* vmptr,int proto, int port, int backlog)
 //	name.sin_family = AF_INET6;
 //	break;
     default:
-	vm->kaya_throw("Protocol not supported (yet)",1);
+	vm->kaya_internalError(8);
     }
 
 #ifdef WIN32
@@ -113,12 +112,12 @@ int net_listen(void* vmptr,int proto, int port, int backlog)
 
     if (bind(sock,(struct sockaddr *)(&name),sizeof(struct sockaddr_in)))
     {
-	vm->kaya_throw("Can't bind to port",1);
+	vm->kaya_internalError(1);
     }
 
     if (listen(sock,backlog))
     {
-	vm->kaya_throw("Can't listen",1);
+	vm->kaya_internalError(2);
     }
 
     return sock;
@@ -133,7 +132,7 @@ void* net_accept(void* vmptr,int socket)
 
     int client = accept(socket,(struct sockaddr *)&addr,&len);
     if (client==-1) {
-	vm->kaya_throw("Accept failed",1);
+	vm->kaya_internalError(3);
     }
     
     ConnInfo *c = new ConnInfo();
@@ -169,7 +168,7 @@ void net_send(void* vmptr,void* conn, wchar_t* rawdata)
         rlen = send(c->cid, data,strlen(data),0);
     } while (errno == EINTR);
     if (rlen<0) {
-	vm->kaya_throw("send failed",1);
+	vm->kaya_internalError(4);
     }
 }
 
@@ -187,7 +186,7 @@ char* net_recv(void* vmptr,void* conn, KayaValue len)
     if (rlen<0) {
 //	cout << rlen << endl;
 //	buf[0]='\0';
-	vm->kaya_throw("nothing to receive",1);
+	vm->kaya_internalError(5);
     }
     return buf;
 }
@@ -223,7 +222,7 @@ bool net_pending(void* vmptr,void* conn, int timeout)
     } while (errno == EINTR);
 
     if (rv==-1) {
-	vm->kaya_throw("select() error",1);	
+	vm->kaya_internalError(6);
     }
     else {
 	return rv;
