@@ -1,12 +1,25 @@
 #ifndef _NETWORK_H // -*-C++-*-
 #define _NETWORK_H
 
+#ifdef WIN32
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#else
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <netdb.h>
+#endif
 
+#ifdef WIN32
+#define FILE_SEPARATOR ''
+#define MSG_WAITALL 0
+#else
+#define FILE_SEPARATOR '/'
+#endif
+
+#include <errno.h>
 #include <gc/gc.h>
 #include <KayaAPI.h>
 
@@ -18,8 +31,10 @@ public:
 };
 
 extern "C" {
+/// Initialise network library (only needed on Windows at the moment)
+    void net_init(void* vm_ptr);
 /// Create a socket and connect to the server. Returns connection info.
-    void* net_connect(void* vmptr,int proto,char* server, int port);
+    void* net_connect(void* vmptr,int proto,wchar_t* server, int port);
 /// Create a socket and start listening. Returns socket id.
     int net_listen(void* vmptr,int proto, int port, int backlog);
 /// Accept a connection. Returns connection info.
@@ -28,7 +43,7 @@ extern "C" {
     void net_shutdown(void* conn);
 /// Close a socket.
     void net_close(void* conn);
-    void net_send(void* vmptr,void* conn, char* data);
+    void net_send(void* vmptr,void* conn, wchar_t* data);
     char* net_recv(void* vmptr,void* conn, KayaValue len);
 /// Get the address of the host in the connection.
     char* net_getaddr(void* conn);
